# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AKS
    from ._models_py3 import AKSSchema
    from ._models_py3 import AKSSchemaProperties
    from ._models_py3 import AccountKeyDatastoreCredentials
    from ._models_py3 import AccountKeyDatastoreSecrets
    from ._models_py3 import AksComputeSecrets
    from ._models_py3 import AksComputeSecretsProperties
    from ._models_py3 import AksNetworkingConfiguration
    from ._models_py3 import AmlCompute
    from ._models_py3 import AmlComputeNodeInformation
    from ._models_py3 import AmlComputeNodesInformation
    from ._models_py3 import AmlComputeProperties
    from ._models_py3 import AmlComputeSchema
    from ._models_py3 import AmlOperation
    from ._models_py3 import AmlOperationDisplay
    from ._models_py3 import AmlOperationListResult
    from ._models_py3 import AmlToken
    from ._models_py3 import AmlUserFeature
    from ._models_py3 import AssetBase
    from ._models_py3 import AssetContainer
    from ._models_py3 import AssetJobInput
    from ._models_py3 import AssetJobOutput
    from ._models_py3 import AssetReferenceBase
    from ._models_py3 import AssignedUser
    from ._models_py3 import AutoForecastHorizon
    from ._models_py3 import AutoMLJob
    from ._models_py3 import AutoMLVertical
    from ._models_py3 import AutoNCrossValidations
    from ._models_py3 import AutoPauseProperties
    from ._models_py3 import AutoScaleProperties
    from ._models_py3 import AutoSeasonality
    from ._models_py3 import AutoTargetLags
    from ._models_py3 import AutoTargetRollingWindowSize
    from ._models_py3 import AzureBlobDatastore
    from ._models_py3 import AzureDataLakeGen1Datastore
    from ._models_py3 import AzureDataLakeGen2Datastore
    from ._models_py3 import AzureFileDatastore
    from ._models_py3 import BanditPolicy
    from ._models_py3 import BatchDeployment
    from ._models_py3 import BatchDeploymentProperties
    from ._models_py3 import BatchDeploymentTrackedResourceArmPaginatedResult
    from ._models_py3 import BatchEndpoint
    from ._models_py3 import BatchEndpointDefaults
    from ._models_py3 import BatchEndpointProperties
    from ._models_py3 import BatchEndpointTrackedResourceArmPaginatedResult
    from ._models_py3 import BatchRetrySettings
    from ._models_py3 import BayesianSamplingAlgorithm
    from ._models_py3 import BuildContext
    from ._models_py3 import CertificateDatastoreCredentials
    from ._models_py3 import CertificateDatastoreSecrets
    from ._models_py3 import Classification
    from ._models_py3 import ClassificationTrainingSettings
    from ._models_py3 import ClusterUpdateParameters
    from ._models_py3 import CodeConfiguration
    from ._models_py3 import CodeContainer
    from ._models_py3 import CodeContainerProperties
    from ._models_py3 import CodeContainerResourceArmPaginatedResult
    from ._models_py3 import CodeVersion
    from ._models_py3 import CodeVersionProperties
    from ._models_py3 import CodeVersionResourceArmPaginatedResult
    from ._models_py3 import ColumnTransformer
    from ._models_py3 import CommandJob
    from ._models_py3 import CommandJobLimits
    from ._models_py3 import ComponentContainer
    from ._models_py3 import ComponentContainerProperties
    from ._models_py3 import ComponentContainerResourceArmPaginatedResult
    from ._models_py3 import ComponentVersion
    from ._models_py3 import ComponentVersionProperties
    from ._models_py3 import ComponentVersionResourceArmPaginatedResult
    from ._models_py3 import Compute
    from ._models_py3 import ComputeInstance
    from ._models_py3 import ComputeInstanceApplication
    from ._models_py3 import ComputeInstanceConnectivityEndpoints
    from ._models_py3 import ComputeInstanceContainer
    from ._models_py3 import ComputeInstanceCreatedBy
    from ._models_py3 import ComputeInstanceDataDisk
    from ._models_py3 import ComputeInstanceDataMount
    from ._models_py3 import ComputeInstanceEnvironmentInfo
    from ._models_py3 import ComputeInstanceLastOperation
    from ._models_py3 import ComputeInstanceProperties
    from ._models_py3 import ComputeInstanceSchema
    from ._models_py3 import ComputeInstanceSshSettings
    from ._models_py3 import ComputeInstanceVersion
    from ._models_py3 import ComputeResource
    from ._models_py3 import ComputeResourceSchema
    from ._models_py3 import ComputeSchedules
    from ._models_py3 import ComputeSecrets
    from ._models_py3 import ComputeStartStopSchedule
    from ._models_py3 import ContainerResourceRequirements
    from ._models_py3 import ContainerResourceSettings
    from ._models_py3 import CosmosDbSettings
    from ._models_py3 import CronTrigger
    from ._models_py3 import CustomForecastHorizon
    from ._models_py3 import CustomModelJobInput
    from ._models_py3 import CustomModelJobOutput
    from ._models_py3 import CustomNCrossValidations
    from ._models_py3 import CustomSeasonality
    from ._models_py3 import CustomTargetLags
    from ._models_py3 import CustomTargetRollingWindowSize
    from ._models_py3 import DataContainer
    from ._models_py3 import DataContainerProperties
    from ._models_py3 import DataContainerResourceArmPaginatedResult
    from ._models_py3 import DataFactory
    from ._models_py3 import DataLakeAnalytics
    from ._models_py3 import DataLakeAnalyticsSchema
    from ._models_py3 import DataLakeAnalyticsSchemaProperties
    from ._models_py3 import DataPathAssetReference
    from ._models_py3 import DataVersionBase
    from ._models_py3 import DataVersionBaseProperties
    from ._models_py3 import DataVersionBaseResourceArmPaginatedResult
    from ._models_py3 import Databricks
    from ._models_py3 import DatabricksComputeSecrets
    from ._models_py3 import DatabricksComputeSecretsProperties
    from ._models_py3 import DatabricksProperties
    from ._models_py3 import DatabricksSchema
    from ._models_py3 import Datastore
    from ._models_py3 import DatastoreCredentials
    from ._models_py3 import DatastoreProperties
    from ._models_py3 import DatastoreResourceArmPaginatedResult
    from ._models_py3 import DatastoreSecrets
    from ._models_py3 import DefaultScaleSettings
    from ._models_py3 import DeploymentLogs
    from ._models_py3 import DeploymentLogsRequest
    from ._models_py3 import DeploymentResourceConfiguration
    from ._models_py3 import DiagnoseRequestProperties
    from ._models_py3 import DiagnoseResponseResult
    from ._models_py3 import DiagnoseResponseResultValue
    from ._models_py3 import DiagnoseResult
    from ._models_py3 import DiagnoseWorkspaceParameters
    from ._models_py3 import DistributionConfiguration
    from ._models_py3 import EarlyTerminationPolicy
    from ._models_py3 import EncryptionKeyVaultProperties
    from ._models_py3 import EncryptionProperty
    from ._models_py3 import EndpointAuthKeys
    from ._models_py3 import EndpointAuthToken
    from ._models_py3 import EndpointDeploymentPropertiesBase
    from ._models_py3 import EndpointPropertiesBase
    from ._models_py3 import EndpointScheduleAction
    from ._models_py3 import EnvironmentContainer
    from ._models_py3 import EnvironmentContainerProperties
    from ._models_py3 import EnvironmentContainerResourceArmPaginatedResult
    from ._models_py3 import EnvironmentVersion
    from ._models_py3 import EnvironmentVersionProperties
    from ._models_py3 import EnvironmentVersionResourceArmPaginatedResult
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorDetail
    from ._models_py3 import ErrorResponse
    from ._models_py3 import EstimatedVMPrice
    from ._models_py3 import EstimatedVMPrices
    from ._models_py3 import ExternalFQDNResponse
    from ._models_py3 import FQDNEndpoint
    from ._models_py3 import FQDNEndpointDetail
    from ._models_py3 import FQDNEndpoints
    from ._models_py3 import FQDNEndpointsProperties
    from ._models_py3 import FeaturizationSettings
    from ._models_py3 import FlavorData
    from ._models_py3 import ForecastHorizon
    from ._models_py3 import Forecasting
    from ._models_py3 import ForecastingSettings
    from ._models_py3 import ForecastingTrainingSettings
    from ._models_py3 import GridSamplingAlgorithm
    from ._models_py3 import HDInsight
    from ._models_py3 import HDInsightProperties
    from ._models_py3 import HDInsightSchema
    from ._models_py3 import IdAssetReference
    from ._models_py3 import IdentityConfiguration
    from ._models_py3 import IdentityForCmk
    from ._models_py3 import ImageClassification
    from ._models_py3 import ImageClassificationBase
    from ._models_py3 import ImageClassificationMultilabel
    from ._models_py3 import ImageInstanceSegmentation
    from ._models_py3 import ImageLimitSettings
    from ._models_py3 import ImageModelDistributionSettings
    from ._models_py3 import ImageModelDistributionSettingsClassification
    from ._models_py3 import ImageModelDistributionSettingsObjectDetection
    from ._models_py3 import ImageModelSettings
    from ._models_py3 import ImageModelSettingsClassification
    from ._models_py3 import ImageModelSettingsObjectDetection
    from ._models_py3 import ImageObjectDetection
    from ._models_py3 import ImageObjectDetectionBase
    from ._models_py3 import ImageSweepSettings
    from ._models_py3 import ImageVertical
    from ._models_py3 import InferenceContainerProperties
    from ._models_py3 import InstanceTypeSchema
    from ._models_py3 import InstanceTypeSchemaResources
    from ._models_py3 import JobBase
    from ._models_py3 import JobBaseProperties
    from ._models_py3 import JobBaseResourceArmPaginatedResult
    from ._models_py3 import JobInput
    from ._models_py3 import JobLimits
    from ._models_py3 import JobOutput
    from ._models_py3 import JobResourceConfiguration
    from ._models_py3 import JobScheduleAction
    from ._models_py3 import JobService
    from ._models_py3 import Kubernetes
    from ._models_py3 import KubernetesOnlineDeployment
    from ._models_py3 import KubernetesProperties
    from ._models_py3 import KubernetesSchema
    from ._models_py3 import ListAmlUserFeatureResult
    from ._models_py3 import ListNotebookKeysResult
    from ._models_py3 import ListStorageAccountKeysResult
    from ._models_py3 import ListUsagesResult
    from ._models_py3 import ListWorkspaceKeysResult
    from ._models_py3 import ListWorkspaceQuotas
    from ._models_py3 import LiteralJobInput
    from ._models_py3 import MLFlowModelJobInput
    from ._models_py3 import MLFlowModelJobOutput
    from ._models_py3 import MLTableData
    from ._models_py3 import MLTableJobInput
    from ._models_py3 import MLTableJobOutput
    from ._models_py3 import ManagedIdentity
    from ._models_py3 import ManagedIdentityAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import ManagedOnlineDeployment
    from ._models_py3 import ManagedServiceIdentity
    from ._models_py3 import MedianStoppingPolicy
    from ._models_py3 import ModelContainer
    from ._models_py3 import ModelContainerProperties
    from ._models_py3 import ModelContainerResourceArmPaginatedResult
    from ._models_py3 import ModelVersion
    from ._models_py3 import ModelVersionProperties
    from ._models_py3 import ModelVersionResourceArmPaginatedResult
    from ._models_py3 import Mpi
    from ._models_py3 import NCrossValidations
    from ._models_py3 import NlpVertical
    from ._models_py3 import NlpVerticalFeaturizationSettings
    from ._models_py3 import NlpVerticalLimitSettings
    from ._models_py3 import NodeStateCounts
    from ._models_py3 import NoneAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import NoneDatastoreCredentials
    from ._models_py3 import NotebookAccessTokenResult
    from ._models_py3 import NotebookPreparationError
    from ._models_py3 import NotebookResourceInfo
    from ._models_py3 import Objective
    from ._models_py3 import OnlineDeployment
    from ._models_py3 import OnlineDeploymentProperties
    from ._models_py3 import OnlineDeploymentTrackedResourceArmPaginatedResult
    from ._models_py3 import OnlineEndpoint
    from ._models_py3 import OnlineEndpointProperties
    from ._models_py3 import OnlineEndpointTrackedResourceArmPaginatedResult
    from ._models_py3 import OnlineRequestSettings
    from ._models_py3 import OnlineScaleSettings
    from ._models_py3 import OutputPathAssetReference
    from ._models_py3 import PATAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import PaginatedComputeResourcesList
    from ._models_py3 import PartialBatchDeployment
    from ._models_py3 import PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties
    from ._models_py3 import PartialManagedServiceIdentity
    from ._models_py3 import PartialMinimalTrackedResource
    from ._models_py3 import PartialMinimalTrackedResourceWithIdentity
    from ._models_py3 import PartialMinimalTrackedResourceWithSku
    from ._models_py3 import PartialSku
    from ._models_py3 import Password
    from ._models_py3 import PersonalComputeInstanceSettings
    from ._models_py3 import PipelineJob
    from ._models_py3 import PrivateEndpoint
    from ._models_py3 import PrivateEndpointConnection
    from ._models_py3 import PrivateEndpointConnectionListResult
    from ._models_py3 import PrivateLinkResource
    from ._models_py3 import PrivateLinkResourceListResult
    from ._models_py3 import PrivateLinkServiceConnectionState
    from ._models_py3 import ProbeSettings
    from ._models_py3 import PyTorch
    from ._models_py3 import QuotaBaseProperties
    from ._models_py3 import QuotaUpdateParameters
    from ._models_py3 import RandomSamplingAlgorithm
    from ._models_py3 import RecurrenceSchedule
    from ._models_py3 import RecurrenceTrigger
    from ._models_py3 import RegenerateEndpointKeysRequest
    from ._models_py3 import RegistryListCredentialsResult
    from ._models_py3 import Regression
    from ._models_py3 import RegressionTrainingSettings
    from ._models_py3 import Resource
    from ._models_py3 import ResourceBase
    from ._models_py3 import ResourceConfiguration
    from ._models_py3 import ResourceId
    from ._models_py3 import ResourceName
    from ._models_py3 import ResourceQuota
    from ._models_py3 import Route
    from ._models_py3 import SASAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import SamplingAlgorithm
    from ._models_py3 import SasDatastoreCredentials
    from ._models_py3 import SasDatastoreSecrets
    from ._models_py3 import ScaleSettings
    from ._models_py3 import ScaleSettingsInformation
    from ._models_py3 import Schedule
    from ._models_py3 import ScheduleActionBase
    from ._models_py3 import ScheduleBase
    from ._models_py3 import ScheduleProperties
    from ._models_py3 import ScheduleResourceArmPaginatedResult
    from ._models_py3 import ScriptReference
    from ._models_py3 import ScriptsToExecute
    from ._models_py3 import Seasonality
    from ._models_py3 import ServiceManagedResourcesSettings
    from ._models_py3 import ServicePrincipalDatastoreCredentials
    from ._models_py3 import ServicePrincipalDatastoreSecrets
    from ._models_py3 import SetupScripts
    from ._models_py3 import SharedPrivateLinkResource
    from ._models_py3 import Sku
    from ._models_py3 import SkuCapacity
    from ._models_py3 import SkuResource
    from ._models_py3 import SkuResourceArmPaginatedResult
    from ._models_py3 import SkuSetting
    from ._models_py3 import SslConfiguration
    from ._models_py3 import StackEnsembleSettings
    from ._models_py3 import SweepJob
    from ._models_py3 import SweepJobLimits
    from ._models_py3 import SynapseSpark
    from ._models_py3 import SynapseSparkProperties
    from ._models_py3 import SystemData
    from ._models_py3 import SystemService
    from ._models_py3 import TableVertical
    from ._models_py3 import TableVerticalFeaturizationSettings
    from ._models_py3 import TableVerticalLimitSettings
    from ._models_py3 import TargetLags
    from ._models_py3 import TargetRollingWindowSize
    from ._models_py3 import TargetUtilizationScaleSettings
    from ._models_py3 import TensorFlow
    from ._models_py3 import TextClassification
    from ._models_py3 import TextClassificationMultilabel
    from ._models_py3 import TextNer
    from ._models_py3 import TrackedResource
    from ._models_py3 import TrainingSettings
    from ._models_py3 import TrialComponent
    from ._models_py3 import TriggerBase
    from ._models_py3 import TritonModelJobInput
    from ._models_py3 import TritonModelJobOutput
    from ._models_py3 import TruncationSelectionPolicy
    from ._models_py3 import UpdateWorkspaceQuotas
    from ._models_py3 import UpdateWorkspaceQuotasResult
    from ._models_py3 import UriFileDataVersion
    from ._models_py3 import UriFileJobInput
    from ._models_py3 import UriFileJobOutput
    from ._models_py3 import UriFolderDataVersion
    from ._models_py3 import UriFolderJobInput
    from ._models_py3 import UriFolderJobOutput
    from ._models_py3 import Usage
    from ._models_py3 import UsageName
    from ._models_py3 import UserAccountCredentials
    from ._models_py3 import UserAssignedIdentity
    from ._models_py3 import UserIdentity
    from ._models_py3 import UsernamePasswordAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import VirtualMachine
    from ._models_py3 import VirtualMachineImage
    from ._models_py3 import VirtualMachineSchema
    from ._models_py3 import VirtualMachineSchemaProperties
    from ._models_py3 import VirtualMachineSecrets
    from ._models_py3 import VirtualMachineSecretsSchema
    from ._models_py3 import VirtualMachineSize
    from ._models_py3 import VirtualMachineSizeListResult
    from ._models_py3 import VirtualMachineSshCredentials
    from ._models_py3 import Workspace
    from ._models_py3 import WorkspaceConnectionManagedIdentity
    from ._models_py3 import WorkspaceConnectionPersonalAccessToken
    from ._models_py3 import WorkspaceConnectionPropertiesV2
    from ._models_py3 import WorkspaceConnectionPropertiesV2BasicResource
    from ._models_py3 import WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult
    from ._models_py3 import WorkspaceConnectionSharedAccessSignature
    from ._models_py3 import WorkspaceConnectionUsernamePassword
    from ._models_py3 import WorkspaceListResult
    from ._models_py3 import WorkspaceUpdateParameters
except (SyntaxError, ImportError):
    from ._models import AKS  # type: ignore
    from ._models import AKSSchema  # type: ignore
    from ._models import AKSSchemaProperties  # type: ignore
    from ._models import AccountKeyDatastoreCredentials  # type: ignore
    from ._models import AccountKeyDatastoreSecrets  # type: ignore
    from ._models import AksComputeSecrets  # type: ignore
    from ._models import AksComputeSecretsProperties  # type: ignore
    from ._models import AksNetworkingConfiguration  # type: ignore
    from ._models import AmlCompute  # type: ignore
    from ._models import AmlComputeNodeInformation  # type: ignore
    from ._models import AmlComputeNodesInformation  # type: ignore
    from ._models import AmlComputeProperties  # type: ignore
    from ._models import AmlComputeSchema  # type: ignore
    from ._models import AmlOperation  # type: ignore
    from ._models import AmlOperationDisplay  # type: ignore
    from ._models import AmlOperationListResult  # type: ignore
    from ._models import AmlToken  # type: ignore
    from ._models import AmlUserFeature  # type: ignore
    from ._models import AssetBase  # type: ignore
    from ._models import AssetContainer  # type: ignore
    from ._models import AssetJobInput  # type: ignore
    from ._models import AssetJobOutput  # type: ignore
    from ._models import AssetReferenceBase  # type: ignore
    from ._models import AssignedUser  # type: ignore
    from ._models import AutoForecastHorizon  # type: ignore
    from ._models import AutoMLJob  # type: ignore
    from ._models import AutoMLVertical  # type: ignore
    from ._models import AutoNCrossValidations  # type: ignore
    from ._models import AutoPauseProperties  # type: ignore
    from ._models import AutoScaleProperties  # type: ignore
    from ._models import AutoSeasonality  # type: ignore
    from ._models import AutoTargetLags  # type: ignore
    from ._models import AutoTargetRollingWindowSize  # type: ignore
    from ._models import AzureBlobDatastore  # type: ignore
    from ._models import AzureDataLakeGen1Datastore  # type: ignore
    from ._models import AzureDataLakeGen2Datastore  # type: ignore
    from ._models import AzureFileDatastore  # type: ignore
    from ._models import BanditPolicy  # type: ignore
    from ._models import BatchDeployment  # type: ignore
    from ._models import BatchDeploymentProperties  # type: ignore
    from ._models import BatchDeploymentTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import BatchEndpoint  # type: ignore
    from ._models import BatchEndpointDefaults  # type: ignore
    from ._models import BatchEndpointProperties  # type: ignore
    from ._models import BatchEndpointTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import BatchRetrySettings  # type: ignore
    from ._models import BayesianSamplingAlgorithm  # type: ignore
    from ._models import BuildContext  # type: ignore
    from ._models import CertificateDatastoreCredentials  # type: ignore
    from ._models import CertificateDatastoreSecrets  # type: ignore
    from ._models import Classification  # type: ignore
    from ._models import ClassificationTrainingSettings  # type: ignore
    from ._models import ClusterUpdateParameters  # type: ignore
    from ._models import CodeConfiguration  # type: ignore
    from ._models import CodeContainer  # type: ignore
    from ._models import CodeContainerProperties  # type: ignore
    from ._models import CodeContainerResourceArmPaginatedResult  # type: ignore
    from ._models import CodeVersion  # type: ignore
    from ._models import CodeVersionProperties  # type: ignore
    from ._models import CodeVersionResourceArmPaginatedResult  # type: ignore
    from ._models import ColumnTransformer  # type: ignore
    from ._models import CommandJob  # type: ignore
    from ._models import CommandJobLimits  # type: ignore
    from ._models import ComponentContainer  # type: ignore
    from ._models import ComponentContainerProperties  # type: ignore
    from ._models import ComponentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ComponentVersion  # type: ignore
    from ._models import ComponentVersionProperties  # type: ignore
    from ._models import ComponentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import Compute  # type: ignore
    from ._models import ComputeInstance  # type: ignore
    from ._models import ComputeInstanceApplication  # type: ignore
    from ._models import ComputeInstanceConnectivityEndpoints  # type: ignore
    from ._models import ComputeInstanceContainer  # type: ignore
    from ._models import ComputeInstanceCreatedBy  # type: ignore
    from ._models import ComputeInstanceDataDisk  # type: ignore
    from ._models import ComputeInstanceDataMount  # type: ignore
    from ._models import ComputeInstanceEnvironmentInfo  # type: ignore
    from ._models import ComputeInstanceLastOperation  # type: ignore
    from ._models import ComputeInstanceProperties  # type: ignore
    from ._models import ComputeInstanceSchema  # type: ignore
    from ._models import ComputeInstanceSshSettings  # type: ignore
    from ._models import ComputeInstanceVersion  # type: ignore
    from ._models import ComputeResource  # type: ignore
    from ._models import ComputeResourceSchema  # type: ignore
    from ._models import ComputeSchedules  # type: ignore
    from ._models import ComputeSecrets  # type: ignore
    from ._models import ComputeStartStopSchedule  # type: ignore
    from ._models import ContainerResourceRequirements  # type: ignore
    from ._models import ContainerResourceSettings  # type: ignore
    from ._models import CosmosDbSettings  # type: ignore
    from ._models import CronTrigger  # type: ignore
    from ._models import CustomForecastHorizon  # type: ignore
    from ._models import CustomModelJobInput  # type: ignore
    from ._models import CustomModelJobOutput  # type: ignore
    from ._models import CustomNCrossValidations  # type: ignore
    from ._models import CustomSeasonality  # type: ignore
    from ._models import CustomTargetLags  # type: ignore
    from ._models import CustomTargetRollingWindowSize  # type: ignore
    from ._models import DataContainer  # type: ignore
    from ._models import DataContainerProperties  # type: ignore
    from ._models import DataContainerResourceArmPaginatedResult  # type: ignore
    from ._models import DataFactory  # type: ignore
    from ._models import DataLakeAnalytics  # type: ignore
    from ._models import DataLakeAnalyticsSchema  # type: ignore
    from ._models import DataLakeAnalyticsSchemaProperties  # type: ignore
    from ._models import DataPathAssetReference  # type: ignore
    from ._models import DataVersionBase  # type: ignore
    from ._models import DataVersionBaseProperties  # type: ignore
    from ._models import DataVersionBaseResourceArmPaginatedResult  # type: ignore
    from ._models import Databricks  # type: ignore
    from ._models import DatabricksComputeSecrets  # type: ignore
    from ._models import DatabricksComputeSecretsProperties  # type: ignore
    from ._models import DatabricksProperties  # type: ignore
    from ._models import DatabricksSchema  # type: ignore
    from ._models import Datastore  # type: ignore
    from ._models import DatastoreCredentials  # type: ignore
    from ._models import DatastoreProperties  # type: ignore
    from ._models import DatastoreResourceArmPaginatedResult  # type: ignore
    from ._models import DatastoreSecrets  # type: ignore
    from ._models import DefaultScaleSettings  # type: ignore
    from ._models import DeploymentLogs  # type: ignore
    from ._models import DeploymentLogsRequest  # type: ignore
    from ._models import DeploymentResourceConfiguration  # type: ignore
    from ._models import DiagnoseRequestProperties  # type: ignore
    from ._models import DiagnoseResponseResult  # type: ignore
    from ._models import DiagnoseResponseResultValue  # type: ignore
    from ._models import DiagnoseResult  # type: ignore
    from ._models import DiagnoseWorkspaceParameters  # type: ignore
    from ._models import DistributionConfiguration  # type: ignore
    from ._models import EarlyTerminationPolicy  # type: ignore
    from ._models import EncryptionKeyVaultProperties  # type: ignore
    from ._models import EncryptionProperty  # type: ignore
    from ._models import EndpointAuthKeys  # type: ignore
    from ._models import EndpointAuthToken  # type: ignore
    from ._models import EndpointDeploymentPropertiesBase  # type: ignore
    from ._models import EndpointPropertiesBase  # type: ignore
    from ._models import EndpointScheduleAction  # type: ignore
    from ._models import EnvironmentContainer  # type: ignore
    from ._models import EnvironmentContainerProperties  # type: ignore
    from ._models import EnvironmentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import EnvironmentVersion  # type: ignore
    from ._models import EnvironmentVersionProperties  # type: ignore
    from ._models import EnvironmentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorDetail  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import EstimatedVMPrice  # type: ignore
    from ._models import EstimatedVMPrices  # type: ignore
    from ._models import ExternalFQDNResponse  # type: ignore
    from ._models import FQDNEndpoint  # type: ignore
    from ._models import FQDNEndpointDetail  # type: ignore
    from ._models import FQDNEndpoints  # type: ignore
    from ._models import FQDNEndpointsProperties  # type: ignore
    from ._models import FeaturizationSettings  # type: ignore
    from ._models import FlavorData  # type: ignore
    from ._models import ForecastHorizon  # type: ignore
    from ._models import Forecasting  # type: ignore
    from ._models import ForecastingSettings  # type: ignore
    from ._models import ForecastingTrainingSettings  # type: ignore
    from ._models import GridSamplingAlgorithm  # type: ignore
    from ._models import HDInsight  # type: ignore
    from ._models import HDInsightProperties  # type: ignore
    from ._models import HDInsightSchema  # type: ignore
    from ._models import IdAssetReference  # type: ignore
    from ._models import IdentityConfiguration  # type: ignore
    from ._models import IdentityForCmk  # type: ignore
    from ._models import ImageClassification  # type: ignore
    from ._models import ImageClassificationBase  # type: ignore
    from ._models import ImageClassificationMultilabel  # type: ignore
    from ._models import ImageInstanceSegmentation  # type: ignore
    from ._models import ImageLimitSettings  # type: ignore
    from ._models import ImageModelDistributionSettings  # type: ignore
    from ._models import ImageModelDistributionSettingsClassification  # type: ignore
    from ._models import ImageModelDistributionSettingsObjectDetection  # type: ignore
    from ._models import ImageModelSettings  # type: ignore
    from ._models import ImageModelSettingsClassification  # type: ignore
    from ._models import ImageModelSettingsObjectDetection  # type: ignore
    from ._models import ImageObjectDetection  # type: ignore
    from ._models import ImageObjectDetectionBase  # type: ignore
    from ._models import ImageSweepSettings  # type: ignore
    from ._models import ImageVertical  # type: ignore
    from ._models import InferenceContainerProperties  # type: ignore
    from ._models import InstanceTypeSchema  # type: ignore
    from ._models import InstanceTypeSchemaResources  # type: ignore
    from ._models import JobBase  # type: ignore
    from ._models import JobBaseProperties  # type: ignore
    from ._models import JobBaseResourceArmPaginatedResult  # type: ignore
    from ._models import JobInput  # type: ignore
    from ._models import JobLimits  # type: ignore
    from ._models import JobOutput  # type: ignore
    from ._models import JobResourceConfiguration  # type: ignore
    from ._models import JobScheduleAction  # type: ignore
    from ._models import JobService  # type: ignore
    from ._models import Kubernetes  # type: ignore
    from ._models import KubernetesOnlineDeployment  # type: ignore
    from ._models import KubernetesProperties  # type: ignore
    from ._models import KubernetesSchema  # type: ignore
    from ._models import ListAmlUserFeatureResult  # type: ignore
    from ._models import ListNotebookKeysResult  # type: ignore
    from ._models import ListStorageAccountKeysResult  # type: ignore
    from ._models import ListUsagesResult  # type: ignore
    from ._models import ListWorkspaceKeysResult  # type: ignore
    from ._models import ListWorkspaceQuotas  # type: ignore
    from ._models import LiteralJobInput  # type: ignore
    from ._models import MLFlowModelJobInput  # type: ignore
    from ._models import MLFlowModelJobOutput  # type: ignore
    from ._models import MLTableData  # type: ignore
    from ._models import MLTableJobInput  # type: ignore
    from ._models import MLTableJobOutput  # type: ignore
    from ._models import ManagedIdentity  # type: ignore
    from ._models import ManagedIdentityAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import ManagedOnlineDeployment  # type: ignore
    from ._models import ManagedServiceIdentity  # type: ignore
    from ._models import MedianStoppingPolicy  # type: ignore
    from ._models import ModelContainer  # type: ignore
    from ._models import ModelContainerProperties  # type: ignore
    from ._models import ModelContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ModelVersion  # type: ignore
    from ._models import ModelVersionProperties  # type: ignore
    from ._models import ModelVersionResourceArmPaginatedResult  # type: ignore
    from ._models import Mpi  # type: ignore
    from ._models import NCrossValidations  # type: ignore
    from ._models import NlpVertical  # type: ignore
    from ._models import NlpVerticalFeaturizationSettings  # type: ignore
    from ._models import NlpVerticalLimitSettings  # type: ignore
    from ._models import NodeStateCounts  # type: ignore
    from ._models import NoneAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import NoneDatastoreCredentials  # type: ignore
    from ._models import NotebookAccessTokenResult  # type: ignore
    from ._models import NotebookPreparationError  # type: ignore
    from ._models import NotebookResourceInfo  # type: ignore
    from ._models import Objective  # type: ignore
    from ._models import OnlineDeployment  # type: ignore
    from ._models import OnlineDeploymentProperties  # type: ignore
    from ._models import OnlineDeploymentTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import OnlineEndpoint  # type: ignore
    from ._models import OnlineEndpointProperties  # type: ignore
    from ._models import OnlineEndpointTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import OnlineRequestSettings  # type: ignore
    from ._models import OnlineScaleSettings  # type: ignore
    from ._models import OutputPathAssetReference  # type: ignore
    from ._models import PATAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import PaginatedComputeResourcesList  # type: ignore
    from ._models import PartialBatchDeployment  # type: ignore
    from ._models import PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties  # type: ignore
    from ._models import PartialManagedServiceIdentity  # type: ignore
    from ._models import PartialMinimalTrackedResource  # type: ignore
    from ._models import PartialMinimalTrackedResourceWithIdentity  # type: ignore
    from ._models import PartialMinimalTrackedResourceWithSku  # type: ignore
    from ._models import PartialSku  # type: ignore
    from ._models import Password  # type: ignore
    from ._models import PersonalComputeInstanceSettings  # type: ignore
    from ._models import PipelineJob  # type: ignore
    from ._models import PrivateEndpoint  # type: ignore
    from ._models import PrivateEndpointConnection  # type: ignore
    from ._models import PrivateEndpointConnectionListResult  # type: ignore
    from ._models import PrivateLinkResource  # type: ignore
    from ._models import PrivateLinkResourceListResult  # type: ignore
    from ._models import PrivateLinkServiceConnectionState  # type: ignore
    from ._models import ProbeSettings  # type: ignore
    from ._models import PyTorch  # type: ignore
    from ._models import QuotaBaseProperties  # type: ignore
    from ._models import QuotaUpdateParameters  # type: ignore
    from ._models import RandomSamplingAlgorithm  # type: ignore
    from ._models import RecurrenceSchedule  # type: ignore
    from ._models import RecurrenceTrigger  # type: ignore
    from ._models import RegenerateEndpointKeysRequest  # type: ignore
    from ._models import RegistryListCredentialsResult  # type: ignore
    from ._models import Regression  # type: ignore
    from ._models import RegressionTrainingSettings  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceBase  # type: ignore
    from ._models import ResourceConfiguration  # type: ignore
    from ._models import ResourceId  # type: ignore
    from ._models import ResourceName  # type: ignore
    from ._models import ResourceQuota  # type: ignore
    from ._models import Route  # type: ignore
    from ._models import SASAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import SamplingAlgorithm  # type: ignore
    from ._models import SasDatastoreCredentials  # type: ignore
    from ._models import SasDatastoreSecrets  # type: ignore
    from ._models import ScaleSettings  # type: ignore
    from ._models import ScaleSettingsInformation  # type: ignore
    from ._models import Schedule  # type: ignore
    from ._models import ScheduleActionBase  # type: ignore
    from ._models import ScheduleBase  # type: ignore
    from ._models import ScheduleProperties  # type: ignore
    from ._models import ScheduleResourceArmPaginatedResult  # type: ignore
    from ._models import ScriptReference  # type: ignore
    from ._models import ScriptsToExecute  # type: ignore
    from ._models import Seasonality  # type: ignore
    from ._models import ServiceManagedResourcesSettings  # type: ignore
    from ._models import ServicePrincipalDatastoreCredentials  # type: ignore
    from ._models import ServicePrincipalDatastoreSecrets  # type: ignore
    from ._models import SetupScripts  # type: ignore
    from ._models import SharedPrivateLinkResource  # type: ignore
    from ._models import Sku  # type: ignore
    from ._models import SkuCapacity  # type: ignore
    from ._models import SkuResource  # type: ignore
    from ._models import SkuResourceArmPaginatedResult  # type: ignore
    from ._models import SkuSetting  # type: ignore
    from ._models import SslConfiguration  # type: ignore
    from ._models import StackEnsembleSettings  # type: ignore
    from ._models import SweepJob  # type: ignore
    from ._models import SweepJobLimits  # type: ignore
    from ._models import SynapseSpark  # type: ignore
    from ._models import SynapseSparkProperties  # type: ignore
    from ._models import SystemData  # type: ignore
    from ._models import SystemService  # type: ignore
    from ._models import TableVertical  # type: ignore
    from ._models import TableVerticalFeaturizationSettings  # type: ignore
    from ._models import TableVerticalLimitSettings  # type: ignore
    from ._models import TargetLags  # type: ignore
    from ._models import TargetRollingWindowSize  # type: ignore
    from ._models import TargetUtilizationScaleSettings  # type: ignore
    from ._models import TensorFlow  # type: ignore
    from ._models import TextClassification  # type: ignore
    from ._models import TextClassificationMultilabel  # type: ignore
    from ._models import TextNer  # type: ignore
    from ._models import TrackedResource  # type: ignore
    from ._models import TrainingSettings  # type: ignore
    from ._models import TrialComponent  # type: ignore
    from ._models import TriggerBase  # type: ignore
    from ._models import TritonModelJobInput  # type: ignore
    from ._models import TritonModelJobOutput  # type: ignore
    from ._models import TruncationSelectionPolicy  # type: ignore
    from ._models import UpdateWorkspaceQuotas  # type: ignore
    from ._models import UpdateWorkspaceQuotasResult  # type: ignore
    from ._models import UriFileDataVersion  # type: ignore
    from ._models import UriFileJobInput  # type: ignore
    from ._models import UriFileJobOutput  # type: ignore
    from ._models import UriFolderDataVersion  # type: ignore
    from ._models import UriFolderJobInput  # type: ignore
    from ._models import UriFolderJobOutput  # type: ignore
    from ._models import Usage  # type: ignore
    from ._models import UsageName  # type: ignore
    from ._models import UserAccountCredentials  # type: ignore
    from ._models import UserAssignedIdentity  # type: ignore
    from ._models import UserIdentity  # type: ignore
    from ._models import UsernamePasswordAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import VirtualMachine  # type: ignore
    from ._models import VirtualMachineImage  # type: ignore
    from ._models import VirtualMachineSchema  # type: ignore
    from ._models import VirtualMachineSchemaProperties  # type: ignore
    from ._models import VirtualMachineSecrets  # type: ignore
    from ._models import VirtualMachineSecretsSchema  # type: ignore
    from ._models import VirtualMachineSize  # type: ignore
    from ._models import VirtualMachineSizeListResult  # type: ignore
    from ._models import VirtualMachineSshCredentials  # type: ignore
    from ._models import Workspace  # type: ignore
    from ._models import WorkspaceConnectionManagedIdentity  # type: ignore
    from ._models import WorkspaceConnectionPersonalAccessToken  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2BasicResource  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult  # type: ignore
    from ._models import WorkspaceConnectionSharedAccessSignature  # type: ignore
    from ._models import WorkspaceConnectionUsernamePassword  # type: ignore
    from ._models import WorkspaceListResult  # type: ignore
    from ._models import WorkspaceUpdateParameters  # type: ignore

from ._azure_machine_learning_workspaces_enums import (
    AllocationState,
    ApplicationSharingPolicy,
    AutoRebuildSetting,
    Autosave,
    BatchLoggingLevel,
    BatchOutputAction,
    BillingCurrency,
    BlockedTransformers,
    Caching,
    ClassificationModels,
    ClassificationMultilabelPrimaryMetrics,
    ClassificationPrimaryMetrics,
    ClusterPurpose,
    ComputeInstanceAuthorizationType,
    ComputeInstanceState,
    ComputePowerAction,
    ComputeType,
    ConnectionAuthType,
    ConnectionCategory,
    ContainerType,
    CreatedByType,
    CredentialsType,
    DataType,
    DatastoreType,
    DeploymentProvisioningState,
    DiagnoseResultLevel,
    DistributionType,
    EarlyTerminationPolicyType,
    EgressPublicNetworkAccessType,
    EncryptionStatus,
    EndpointAuthMode,
    EndpointComputeType,
    EndpointProvisioningState,
    EnvironmentType,
    FeatureLags,
    FeaturizationMode,
    ForecastHorizonMode,
    ForecastingModels,
    ForecastingPrimaryMetrics,
    Goal,
    IdentityConfigurationType,
    InputDeliveryMode,
    InstanceSegmentationPrimaryMetrics,
    JobInputType,
    JobLimitsType,
    JobOutputType,
    JobStatus,
    JobType,
    KeyType,
    LearningRateScheduler,
    ListViewType,
    LoadBalancerType,
    LogVerbosity,
    ManagedServiceIdentityType,
    ModelSize,
    MountAction,
    MountState,
    NCrossValidationsMode,
    Network,
    NodeState,
    ObjectDetectionPrimaryMetrics,
    OperatingSystemType,
    OperationName,
    OperationStatus,
    OperationTrigger,
    OrderString,
    OsType,
    OutputDeliveryMode,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    ProvisioningState,
    ProvisioningStatus,
    PublicNetworkAccess,
    PublicNetworkAccessType,
    QuotaUnit,
    RandomSamplingAlgorithmRule,
    RecurrenceFrequency,
    ReferenceType,
    RegressionModels,
    RegressionPrimaryMetrics,
    RemoteLoginPortPublicAccess,
    SamplingAlgorithmType,
    ScaleType,
    ScheduleActionType,
    ScheduleListViewType,
    ScheduleProvisioningState,
    ScheduleProvisioningStatus,
    ScheduleStatus,
    SeasonalityMode,
    SecretsType,
    ServiceDataAccessAuthIdentity,
    ShortSeriesHandlingConfiguration,
    SkuScaleType,
    SkuTier,
    SourceType,
    SshPublicAccess,
    SslConfigStatus,
    StackMetaLearnerType,
    Status,
    StochasticOptimizer,
    StorageAccountType,
    TargetAggregationFunction,
    TargetLagsMode,
    TargetRollingWindowSizeMode,
    TaskType,
    TriggerType,
    UnderlyingResourceAction,
    UnitOfMeasure,
    UsageUnit,
    UseStl,
    VMPriceOSType,
    VMTier,
    ValidationMetricType,
    ValueFormat,
    VmPriority,
    WeekDay,
)

__all__ = [
    'AKS',
    'AKSSchema',
    'AKSSchemaProperties',
    'AccountKeyDatastoreCredentials',
    'AccountKeyDatastoreSecrets',
    'AksComputeSecrets',
    'AksComputeSecretsProperties',
    'AksNetworkingConfiguration',
    'AmlCompute',
    'AmlComputeNodeInformation',
    'AmlComputeNodesInformation',
    'AmlComputeProperties',
    'AmlComputeSchema',
    'AmlOperation',
    'AmlOperationDisplay',
    'AmlOperationListResult',
    'AmlToken',
    'AmlUserFeature',
    'AssetBase',
    'AssetContainer',
    'AssetJobInput',
    'AssetJobOutput',
    'AssetReferenceBase',
    'AssignedUser',
    'AutoForecastHorizon',
    'AutoMLJob',
    'AutoMLVertical',
    'AutoNCrossValidations',
    'AutoPauseProperties',
    'AutoScaleProperties',
    'AutoSeasonality',
    'AutoTargetLags',
    'AutoTargetRollingWindowSize',
    'AzureBlobDatastore',
    'AzureDataLakeGen1Datastore',
    'AzureDataLakeGen2Datastore',
    'AzureFileDatastore',
    'BanditPolicy',
    'BatchDeployment',
    'BatchDeploymentProperties',
    'BatchDeploymentTrackedResourceArmPaginatedResult',
    'BatchEndpoint',
    'BatchEndpointDefaults',
    'BatchEndpointProperties',
    'BatchEndpointTrackedResourceArmPaginatedResult',
    'BatchRetrySettings',
    'BayesianSamplingAlgorithm',
    'BuildContext',
    'CertificateDatastoreCredentials',
    'CertificateDatastoreSecrets',
    'Classification',
    'ClassificationTrainingSettings',
    'ClusterUpdateParameters',
    'CodeConfiguration',
    'CodeContainer',
    'CodeContainerProperties',
    'CodeContainerResourceArmPaginatedResult',
    'CodeVersion',
    'CodeVersionProperties',
    'CodeVersionResourceArmPaginatedResult',
    'ColumnTransformer',
    'CommandJob',
    'CommandJobLimits',
    'ComponentContainer',
    'ComponentContainerProperties',
    'ComponentContainerResourceArmPaginatedResult',
    'ComponentVersion',
    'ComponentVersionProperties',
    'ComponentVersionResourceArmPaginatedResult',
    'Compute',
    'ComputeInstance',
    'ComputeInstanceApplication',
    'ComputeInstanceConnectivityEndpoints',
    'ComputeInstanceContainer',
    'ComputeInstanceCreatedBy',
    'ComputeInstanceDataDisk',
    'ComputeInstanceDataMount',
    'ComputeInstanceEnvironmentInfo',
    'ComputeInstanceLastOperation',
    'ComputeInstanceProperties',
    'ComputeInstanceSchema',
    'ComputeInstanceSshSettings',
    'ComputeInstanceVersion',
    'ComputeResource',
    'ComputeResourceSchema',
    'ComputeSchedules',
    'ComputeSecrets',
    'ComputeStartStopSchedule',
    'ContainerResourceRequirements',
    'ContainerResourceSettings',
    'CosmosDbSettings',
    'CronTrigger',
    'CustomForecastHorizon',
    'CustomModelJobInput',
    'CustomModelJobOutput',
    'CustomNCrossValidations',
    'CustomSeasonality',
    'CustomTargetLags',
    'CustomTargetRollingWindowSize',
    'DataContainer',
    'DataContainerProperties',
    'DataContainerResourceArmPaginatedResult',
    'DataFactory',
    'DataLakeAnalytics',
    'DataLakeAnalyticsSchema',
    'DataLakeAnalyticsSchemaProperties',
    'DataPathAssetReference',
    'DataVersionBase',
    'DataVersionBaseProperties',
    'DataVersionBaseResourceArmPaginatedResult',
    'Databricks',
    'DatabricksComputeSecrets',
    'DatabricksComputeSecretsProperties',
    'DatabricksProperties',
    'DatabricksSchema',
    'Datastore',
    'DatastoreCredentials',
    'DatastoreProperties',
    'DatastoreResourceArmPaginatedResult',
    'DatastoreSecrets',
    'DefaultScaleSettings',
    'DeploymentLogs',
    'DeploymentLogsRequest',
    'DeploymentResourceConfiguration',
    'DiagnoseRequestProperties',
    'DiagnoseResponseResult',
    'DiagnoseResponseResultValue',
    'DiagnoseResult',
    'DiagnoseWorkspaceParameters',
    'DistributionConfiguration',
    'EarlyTerminationPolicy',
    'EncryptionKeyVaultProperties',
    'EncryptionProperty',
    'EndpointAuthKeys',
    'EndpointAuthToken',
    'EndpointDeploymentPropertiesBase',
    'EndpointPropertiesBase',
    'EndpointScheduleAction',
    'EnvironmentContainer',
    'EnvironmentContainerProperties',
    'EnvironmentContainerResourceArmPaginatedResult',
    'EnvironmentVersion',
    'EnvironmentVersionProperties',
    'EnvironmentVersionResourceArmPaginatedResult',
    'ErrorAdditionalInfo',
    'ErrorDetail',
    'ErrorResponse',
    'EstimatedVMPrice',
    'EstimatedVMPrices',
    'ExternalFQDNResponse',
    'FQDNEndpoint',
    'FQDNEndpointDetail',
    'FQDNEndpoints',
    'FQDNEndpointsProperties',
    'FeaturizationSettings',
    'FlavorData',
    'ForecastHorizon',
    'Forecasting',
    'ForecastingSettings',
    'ForecastingTrainingSettings',
    'GridSamplingAlgorithm',
    'HDInsight',
    'HDInsightProperties',
    'HDInsightSchema',
    'IdAssetReference',
    'IdentityConfiguration',
    'IdentityForCmk',
    'ImageClassification',
    'ImageClassificationBase',
    'ImageClassificationMultilabel',
    'ImageInstanceSegmentation',
    'ImageLimitSettings',
    'ImageModelDistributionSettings',
    'ImageModelDistributionSettingsClassification',
    'ImageModelDistributionSettingsObjectDetection',
    'ImageModelSettings',
    'ImageModelSettingsClassification',
    'ImageModelSettingsObjectDetection',
    'ImageObjectDetection',
    'ImageObjectDetectionBase',
    'ImageSweepSettings',
    'ImageVertical',
    'InferenceContainerProperties',
    'InstanceTypeSchema',
    'InstanceTypeSchemaResources',
    'JobBase',
    'JobBaseProperties',
    'JobBaseResourceArmPaginatedResult',
    'JobInput',
    'JobLimits',
    'JobOutput',
    'JobResourceConfiguration',
    'JobScheduleAction',
    'JobService',
    'Kubernetes',
    'KubernetesOnlineDeployment',
    'KubernetesProperties',
    'KubernetesSchema',
    'ListAmlUserFeatureResult',
    'ListNotebookKeysResult',
    'ListStorageAccountKeysResult',
    'ListUsagesResult',
    'ListWorkspaceKeysResult',
    'ListWorkspaceQuotas',
    'LiteralJobInput',
    'MLFlowModelJobInput',
    'MLFlowModelJobOutput',
    'MLTableData',
    'MLTableJobInput',
    'MLTableJobOutput',
    'ManagedIdentity',
    'ManagedIdentityAuthTypeWorkspaceConnectionProperties',
    'ManagedOnlineDeployment',
    'ManagedServiceIdentity',
    'MedianStoppingPolicy',
    'ModelContainer',
    'ModelContainerProperties',
    'ModelContainerResourceArmPaginatedResult',
    'ModelVersion',
    'ModelVersionProperties',
    'ModelVersionResourceArmPaginatedResult',
    'Mpi',
    'NCrossValidations',
    'NlpVertical',
    'NlpVerticalFeaturizationSettings',
    'NlpVerticalLimitSettings',
    'NodeStateCounts',
    'NoneAuthTypeWorkspaceConnectionProperties',
    'NoneDatastoreCredentials',
    'NotebookAccessTokenResult',
    'NotebookPreparationError',
    'NotebookResourceInfo',
    'Objective',
    'OnlineDeployment',
    'OnlineDeploymentProperties',
    'OnlineDeploymentTrackedResourceArmPaginatedResult',
    'OnlineEndpoint',
    'OnlineEndpointProperties',
    'OnlineEndpointTrackedResourceArmPaginatedResult',
    'OnlineRequestSettings',
    'OnlineScaleSettings',
    'OutputPathAssetReference',
    'PATAuthTypeWorkspaceConnectionProperties',
    'PaginatedComputeResourcesList',
    'PartialBatchDeployment',
    'PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties',
    'PartialManagedServiceIdentity',
    'PartialMinimalTrackedResource',
    'PartialMinimalTrackedResourceWithIdentity',
    'PartialMinimalTrackedResourceWithSku',
    'PartialSku',
    'Password',
    'PersonalComputeInstanceSettings',
    'PipelineJob',
    'PrivateEndpoint',
    'PrivateEndpointConnection',
    'PrivateEndpointConnectionListResult',
    'PrivateLinkResource',
    'PrivateLinkResourceListResult',
    'PrivateLinkServiceConnectionState',
    'ProbeSettings',
    'PyTorch',
    'QuotaBaseProperties',
    'QuotaUpdateParameters',
    'RandomSamplingAlgorithm',
    'RecurrenceSchedule',
    'RecurrenceTrigger',
    'RegenerateEndpointKeysRequest',
    'RegistryListCredentialsResult',
    'Regression',
    'RegressionTrainingSettings',
    'Resource',
    'ResourceBase',
    'ResourceConfiguration',
    'ResourceId',
    'ResourceName',
    'ResourceQuota',
    'Route',
    'SASAuthTypeWorkspaceConnectionProperties',
    'SamplingAlgorithm',
    'SasDatastoreCredentials',
    'SasDatastoreSecrets',
    'ScaleSettings',
    'ScaleSettingsInformation',
    'Schedule',
    'ScheduleActionBase',
    'ScheduleBase',
    'ScheduleProperties',
    'ScheduleResourceArmPaginatedResult',
    'ScriptReference',
    'ScriptsToExecute',
    'Seasonality',
    'ServiceManagedResourcesSettings',
    'ServicePrincipalDatastoreCredentials',
    'ServicePrincipalDatastoreSecrets',
    'SetupScripts',
    'SharedPrivateLinkResource',
    'Sku',
    'SkuCapacity',
    'SkuResource',
    'SkuResourceArmPaginatedResult',
    'SkuSetting',
    'SslConfiguration',
    'StackEnsembleSettings',
    'SweepJob',
    'SweepJobLimits',
    'SynapseSpark',
    'SynapseSparkProperties',
    'SystemData',
    'SystemService',
    'TableVertical',
    'TableVerticalFeaturizationSettings',
    'TableVerticalLimitSettings',
    'TargetLags',
    'TargetRollingWindowSize',
    'TargetUtilizationScaleSettings',
    'TensorFlow',
    'TextClassification',
    'TextClassificationMultilabel',
    'TextNer',
    'TrackedResource',
    'TrainingSettings',
    'TrialComponent',
    'TriggerBase',
    'TritonModelJobInput',
    'TritonModelJobOutput',
    'TruncationSelectionPolicy',
    'UpdateWorkspaceQuotas',
    'UpdateWorkspaceQuotasResult',
    'UriFileDataVersion',
    'UriFileJobInput',
    'UriFileJobOutput',
    'UriFolderDataVersion',
    'UriFolderJobInput',
    'UriFolderJobOutput',
    'Usage',
    'UsageName',
    'UserAccountCredentials',
    'UserAssignedIdentity',
    'UserIdentity',
    'UsernamePasswordAuthTypeWorkspaceConnectionProperties',
    'VirtualMachine',
    'VirtualMachineImage',
    'VirtualMachineSchema',
    'VirtualMachineSchemaProperties',
    'VirtualMachineSecrets',
    'VirtualMachineSecretsSchema',
    'VirtualMachineSize',
    'VirtualMachineSizeListResult',
    'VirtualMachineSshCredentials',
    'Workspace',
    'WorkspaceConnectionManagedIdentity',
    'WorkspaceConnectionPersonalAccessToken',
    'WorkspaceConnectionPropertiesV2',
    'WorkspaceConnectionPropertiesV2BasicResource',
    'WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult',
    'WorkspaceConnectionSharedAccessSignature',
    'WorkspaceConnectionUsernamePassword',
    'WorkspaceListResult',
    'WorkspaceUpdateParameters',
    'AllocationState',
    'ApplicationSharingPolicy',
    'AutoRebuildSetting',
    'Autosave',
    'BatchLoggingLevel',
    'BatchOutputAction',
    'BillingCurrency',
    'BlockedTransformers',
    'Caching',
    'ClassificationModels',
    'ClassificationMultilabelPrimaryMetrics',
    'ClassificationPrimaryMetrics',
    'ClusterPurpose',
    'ComputeInstanceAuthorizationType',
    'ComputeInstanceState',
    'ComputePowerAction',
    'ComputeType',
    'ConnectionAuthType',
    'ConnectionCategory',
    'ContainerType',
    'CreatedByType',
    'CredentialsType',
    'DataType',
    'DatastoreType',
    'DeploymentProvisioningState',
    'DiagnoseResultLevel',
    'DistributionType',
    'EarlyTerminationPolicyType',
    'EgressPublicNetworkAccessType',
    'EncryptionStatus',
    'EndpointAuthMode',
    'EndpointComputeType',
    'EndpointProvisioningState',
    'EnvironmentType',
    'FeatureLags',
    'FeaturizationMode',
    'ForecastHorizonMode',
    'ForecastingModels',
    'ForecastingPrimaryMetrics',
    'Goal',
    'IdentityConfigurationType',
    'InputDeliveryMode',
    'InstanceSegmentationPrimaryMetrics',
    'JobInputType',
    'JobLimitsType',
    'JobOutputType',
    'JobStatus',
    'JobType',
    'KeyType',
    'LearningRateScheduler',
    'ListViewType',
    'LoadBalancerType',
    'LogVerbosity',
    'ManagedServiceIdentityType',
    'ModelSize',
    'MountAction',
    'MountState',
    'NCrossValidationsMode',
    'Network',
    'NodeState',
    'ObjectDetectionPrimaryMetrics',
    'OperatingSystemType',
    'OperationName',
    'OperationStatus',
    'OperationTrigger',
    'OrderString',
    'OsType',
    'OutputDeliveryMode',
    'PrivateEndpointConnectionProvisioningState',
    'PrivateEndpointServiceConnectionStatus',
    'ProvisioningState',
    'ProvisioningStatus',
    'PublicNetworkAccess',
    'PublicNetworkAccessType',
    'QuotaUnit',
    'RandomSamplingAlgorithmRule',
    'RecurrenceFrequency',
    'ReferenceType',
    'RegressionModels',
    'RegressionPrimaryMetrics',
    'RemoteLoginPortPublicAccess',
    'SamplingAlgorithmType',
    'ScaleType',
    'ScheduleActionType',
    'ScheduleListViewType',
    'ScheduleProvisioningState',
    'ScheduleProvisioningStatus',
    'ScheduleStatus',
    'SeasonalityMode',
    'SecretsType',
    'ServiceDataAccessAuthIdentity',
    'ShortSeriesHandlingConfiguration',
    'SkuScaleType',
    'SkuTier',
    'SourceType',
    'SshPublicAccess',
    'SslConfigStatus',
    'StackMetaLearnerType',
    'Status',
    'StochasticOptimizer',
    'StorageAccountType',
    'TargetAggregationFunction',
    'TargetLagsMode',
    'TargetRollingWindowSizeMode',
    'TaskType',
    'TriggerType',
    'UnderlyingResourceAction',
    'UnitOfMeasure',
    'UsageUnit',
    'UseStl',
    'VMPriceOSType',
    'VMTier',
    'ValidationMetricType',
    'ValueFormat',
    'VmPriority',
    'WeekDay',
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
# pylint: disable=protected-access, redefined-builtin
# disable redefined-builtin to use globals/locals as argument name

# Attribute on customized group class to mark a value type as a group of inputs/outputs.
import threading
import functools
from typing import Any, Callable, Dict, List, Optional, Type, TypeVar, Union

from azure.ai.ml import Input, Output
from azure.ai.ml.constants._component import IOConstants
from azure.ai.ml.entities._inputs_outputs import GroupInput, _get_param_with_standard_annotation
from azure.ai.ml.entities._inputs_outputs.utils import Annotation

T = TypeVar("T")


def group(_cls: Type[T]) -> Type[T]:
    """Group decorator to make user-defined class as a group of inputs/outputs.

    Usage:
        Group a set of component inputs make component configuration easier.

        e.g. Define a group

        .. code-block:: python

            # define a group
            @dsl.group
            class ParamClass:
                str_param: str
                int_param: int = 1


            # see the help of auto-gen __init__ function
            help(ParamClass.__init__)

        e.g. Define a group with inheritance

        .. code-block:: python

            # define the parent group
            @dsl.group
            class ParentClass:
                str_param: str
                int_param: int = 1


            @dsl.group
            class GroupClass(ParentClass):
                float_param: float
                str_param: str = "test"


            # see the help of auto-gen __init__ function
            help(GroupClass.__init__)

        e.g. Define a multi-level group

        .. code-block:: python

            # define a sub group
            @dsl.group
            class SubGroupClass:
                str_param: str
                int_param: int = 1


            @dsl.group
            class ParentClass:
                param_group1: SubGroupClass
                # declare a sub group field with default
                param_group2: SubGroupClass = SubGroupClass(str_param="test")


            # see the help of auto-gen __init__ function
            help(ParentClass.__init__)

        e.g. Link and assign Group

        .. code-block:: python

            # create a sub group value
            my_param_group = SubGroupClass(str_param="dataset", int_param=2)

            # create a parent group value
            my_parent_group = ParentClass(param_group1=my_param_group)


            # option 1. use annotation to declare the input is a group.
            @pipeline
            def pipeline_func(params: SubGroupClass):
                component = component_func(
                    string_parameter=params.str_param, int_parameter=params.int_param
                )
                return component.outputs


            # create a pipeline instance
            pipeline = pipeline_func(my_param_group)


            # option 2. use default of input to declare itself a group.
            @pipeline
            def pipeline_func(params=my_param_group):
                component = component_func(
                    string_parameter=params.str_param, int_parameter=params.int_param
                )
                return component.outputs


            # create a pipeline instance
            pipeline = pipeline_func()


            # use multi-level group in pipeline.
            @pipeline
            def parent_func(params: ParentClass):
                # pass sub group to sub pipeline.
                pipeline = pipeline_func(params.param_group1)
                return pipeline.outputs

    Supported Types:
        * Primitive type: int, str, float, bool
            * Declare with python type annotation: `param: int`
            * Declare with Input annotation: `param: Input(type='integer', min=1, max=5)`

        * For sub group class
            * Sub group class should be decorated with `dsl.group`

    Restrictions:
        * Each group member's name must be public (not start with '_').
        * When use group as a pipeline input, user **MUST** write the type annotation
          or give it a non-None default value to infer the group class.

    :param _cls: The class to decorate
    :type _cls: Type[T]
    :return: The decorated class
    :rtype: Type[T]
    """

    T2 = TypeVar("T2")

    def _create_fn(
        name: str,
        args: Union[List, str],
        body: Union[List, str],
        *,
        globals: Optional[Dict[str, Any]] = None,
        locals: Optional[Dict[str, Any]] = None,
        return_type: Optional[Type[T2]],
    ) -> Callable[..., T2]:
        """To generate function in class.

        :param name: The name of the new function
        :type name: str
        :param args: The parameter names of the new function
        :type args: List[str]
        :param body: The source code of the body of the new function
        :type body: List[str]
        :keyword globals: The global variables to make available to the new function.
        :paramtype globals: Dict[str, Any]
        :keyword locals: The local variables to make available to the new function
        :paramtype locals: Dict[str, Any]
        :keyword return_type: The return type of the new function
        :paramtype return_type: Type[T2]
        :return: The created function
        :rtype: Callable[..., T2]
        """
        # Reference: Source code of dataclasses.dataclass
        # Doc link: https://docs.python.org/3/library/dataclasses.html
        # Reference code link:
        # https://github.com/python/cpython/blob/17b16e13bb444001534ed6fccb459084596c8bcf/Lib/dataclasses.py#L412
        # Note that we mutate locals when exec() is called
        if locals is None:
            locals = {}
        if "BUILTINS" not in locals:
            import builtins

            locals["BUILTINS"] = builtins
        locals["_return_type"] = return_type
        return_annotation = "->_return_type"
        args = ",".join(args)
        body = "\n".join(f"  {b}" for b in body)
        # Compute the text of the entire function.
        txt = f" def {name}({args}){return_annotation}:\n{body}"
        local_vars = ", ".join(locals.keys())
        txt = f"def __create_fn__({local_vars}):\n{txt}\n return {name}"
        ns: Dict = {}
        exec(txt, globals, ns)  # pylint: disable=exec-used # nosec
        res: Callable = ns["__create_fn__"](**locals)
        return res

    def _create_init_fn(  # pylint: disable=unused-argument
        cls: Type[T], fields: Dict[str, Union[Annotation, Input, Output]]
    ) -> Callable[..., None]:
        """Generate the __init__ function for user-defined class.

        :param cls: The class to update
        :type cls: Type[T]
        :param fields: The fields
        :type fields: Dict[str, Union[Annotation, Input, Output]]
        :return: The __init__ function
        :rtype: Callable[..., None]
        """

        # Reference code link:
        # https://github.com/python/cpython/blob/17b16e13bb444001534ed6fccb459084596c8bcf/Lib/dataclasses.py#L523
        def _get_data_type_from_annotation(anno: Any) -> Any:
            if isinstance(anno, GroupInput):
                return anno._group_class
            # keep original annotation for Outputs
            if isinstance(anno, Output):
                return anno
            try:
                # convert to primitive type annotation if possible
                return IOConstants.PRIMITIVE_STR_2_TYPE[anno.type]
            except KeyError:
                # otherwise, keep original annotation
                return anno

        def _get_default(key: str) -> Any:
            # will set None as default value when default not exist so won't need to reorder the init params
            val = fields[key]
            if val is not None and hasattr(val, "default"):
                return val.default
            return None

        locals = {f"_type_{key}": _get_data_type_from_annotation(val) for key, val in fields.items()}
        # Collect field defaults if val is parameter and is optional
        defaults = {f"_default_{key}": _get_default(key) for key, val in fields.items()}
        locals.update(defaults)
        # Ban positional init as we reordered the parameter.
        _init_param = ["self", "*"]
        for key in fields:
            _init_param.append(f"{key}:_type_{key}=_default_{key}")

        body_lines = [f"self.{key}={key}" for key in fields]
        # If no body lines, use 'pass'.
        if not body_lines:
            body_lines = ["pass"]
        return _create_fn("__init__", _init_param, body_lines, locals=locals, return_type=None)

    def _create_repr_fn(fields: Dict[str, Union[Annotation, Input, Output]]) -> Callable[..., str]:
        """Generate the __repr__ function for user-defined class.

        :param fields: The fields
        :type fields: Dict[str, Union[Annotation, Input, Output]]
        :return: The __repr__ function
        :rtype: Callable[..., str]
        """
        # Reference code link:
        # https://github.com/python/cpython/blob/17b16e13bb444001534ed6fccb459084596c8bcf/Lib/dataclasses.py#L582
        fn = _create_fn(
            "__repr__",
            [
                "self",
            ],
            ['return self.__class__.__qualname__ + f"(' + ", ".join([f"{k}={{self.{k}!r}}" for k in fields]) + ')"'],
            return_type=str,
        )

        # This function's logic is copied from "recursive_repr" function in
        # reprlib module to avoid dependency.
        def _recursive_repr(user_function: Any) -> Any:
            # Decorator to make a repr function return "..." for a recursive
            # call.
            repr_running = set()

            @functools.wraps(user_function)
            def wrapper(self: Any) -> Any:
                key = id(self), threading.get_ident()
                if key in repr_running:
                    return "..."
                repr_running.add(key)
                try:
                    result = user_function(self)
                finally:
                    repr_running.discard(key)
                return result

            return wrapper

        res: Callable = _recursive_repr(fn)
        return res

    def _process_class(cls: Type[T], all_fields: Dict[str, Union[Annotation, Input, Output]]) -> Type[T]:
        setattr(cls, "__init__", _create_init_fn(cls, all_fields))
        setattr(cls, "__repr__", _create_repr_fn(all_fields))
        return cls

    def _wrap(cls: Type[T]) -> Type[T]:
        all_fields = _get_param_with_standard_annotation(cls)
        # Set group info on cls
        setattr(cls, IOConstants.GROUP_ATTR_NAME, GroupInput(all_fields, _group_class=cls))
        # Add init, repr, eq to class with user-defined annotation type
        return _process_class(cls, all_fields)

    return _wrap(_cls)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import re
from os import PathLike
from pathlib import Path
from typing import Dict, Optional, Union

from marshmallow.exceptions import ValidationError as SchemaValidationError

from azure.ai.ml._artifacts._artifact_utilities import (
    _check_and_upload_path,
    _get_datastore_name,
    _get_snapshot_path_info,
    get_datastore_info,
)
from azure.ai.ml._artifacts._constants import (
    ASSET_PATH_ERROR,
    CHANGED_ASSET_PATH_MSG,
    CHANGED_ASSET_PATH_MSG_NO_PERSONAL_DATA,
)
from azure.ai.ml._exception_helper import log_and_raise_error
from azure.ai.ml._restclient.v2021_10_01_dataplanepreview import (
    AzureMachineLearningWorkspaces as ServiceClient102021Dataplane,
)
from azure.ai.ml._restclient.v2022_10_01_preview import AzureMachineLearningWorkspaces as ServiceClient102022
from azure.ai.ml._restclient.v2023_04_01 import AzureMachineLearningWorkspaces as ServiceClient042023
from azure.ai.ml._scope_dependent_operations import OperationConfig, OperationScope, _ScopeDependentOperations
from azure.ai.ml._telemetry import ActivityType, monitor_with_activity
from azure.ai.ml._utils._asset_utils import (
    _get_existing_asset_name_and_version,
    get_content_hash_version,
    get_storage_info_for_non_registry_asset,
)
from azure.ai.ml._utils._logger_utils import OpsLogger
from azure.ai.ml._utils._registry_utils import get_asset_body_for_registry_storage, get_sas_uri_for_registry_asset
from azure.ai.ml._utils._storage_utils import get_storage_client
from azure.ai.ml.entities._assets import Code
from azure.ai.ml.exceptions import (
    AssetPathException,
    ErrorCategory,
    ErrorTarget,
    ValidationErrorType,
    ValidationException,
)
from azure.ai.ml.operations._datastore_operations import DatastoreOperations
from azure.core.exceptions import HttpResponseError

# pylint: disable=protected-access


ops_logger = OpsLogger(__name__)
module_logger = ops_logger.module_logger


class CodeOperations(_ScopeDependentOperations):
    """Represents a client for performing operations on code assets.

    You should not instantiate this class directly. Instead, you should create MLClient and use this client via the
    property MLClient.code

    :param operation_scope: Scope variables for the operations classes of an MLClient object.
    :type operation_scope: ~azure.ai.ml._scope_dependent_operations.OperationScope
    :param operation_config: Common configuration for operations classes of an MLClient object.
    :type operation_config: ~azure.ai.ml._scope_dependent_operations.OperationConfig
    :param service_client: Service client to allow end users to operate on Azure Machine Learning Workspace resources.
    :type service_client: typing.Union[
        ~azure.ai.ml._restclient.v2022_10_01_preview._azure_machine_learning_workspaces.AzureMachineLearningWorkspaces,
        ~azure.ai.ml._restclient.v2021_10_01_dataplanepreview._azure_machine_learning_workspaces.
        AzureMachineLearningWorkspaces,
        ~azure.ai.ml._restclient.v2023_04_01._azure_machine_learning_workspaces.AzureMachineLearningWorkspaces]
    :param datastore_operations: Represents a client for performing operations on Datastores.
    :type datastore_operations: ~azure.ai.ml.operations._datastore_operations.DatastoreOperations
    """

    def __init__(
        self,
        operation_scope: OperationScope,
        operation_config: OperationConfig,
        service_client: Union[ServiceClient102022, ServiceClient102021Dataplane, ServiceClient042023],
        datastore_operations: DatastoreOperations,
        **kwargs: Dict,
    ):
        super(CodeOperations, self).__init__(operation_scope, operation_config)
        ops_logger.update_filter()
        self._service_client = service_client
        self._version_operation = service_client.code_versions
        self._container_operation = service_client.code_containers
        self._datastore_operation = datastore_operations
        self._init_kwargs = kwargs

    @monitor_with_activity(ops_logger, "Code.CreateOrUpdate", ActivityType.PUBLICAPI)
    def create_or_update(self, code: Code) -> Code:
        """Returns created or updated code asset.

        If not already in storage, asset will be uploaded to the workspace's default datastore.

        :param code: Code asset object.
        :type code: Code
        :raises ~azure.ai.ml.exceptions.AssetPathException: Raised when the Code artifact path is
            already linked to another asset
        :raises ~azure.ai.ml.exceptions.ValidationException: Raised if Code cannot be successfully validated.
            Details will be provided in the error message.
        :raises ~azure.ai.ml.exceptions.EmptyDirectoryError: Raised if local path provided points to an empty directory.
        :return: Code asset object.
        :rtype: ~azure.ai.ml.entities.Code

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START code_operations_create_or_update]
                :end-before: [END code_operations_create_or_update]
                :language: python
                :dedent: 8
                :caption: Create code asset example.
        """
        try:
            name = code.name
            version = code.version
            sas_uri = None
            blob_uri = None

            if self._registry_name:
                sas_uri = get_sas_uri_for_registry_asset(
                    service_client=self._service_client,
                    name=name,
                    version=version,
                    resource_group=self._resource_group_name,
                    registry=self._registry_name,
                    body=get_asset_body_for_registry_storage(self._registry_name, "codes", name, version),
                )
            else:
                snapshot_path_info = _get_snapshot_path_info(code)
                if snapshot_path_info:
                    _, _, asset_hash = snapshot_path_info
                    existing_assets = list(
                        self._version_operation.list(
                            resource_group_name=self._resource_group_name,
                            workspace_name=self._workspace_name,
                            name=name,
                            hash=asset_hash,
                            hash_version=str(get_content_hash_version()),
                        )
                    )

                    if len(existing_assets) > 0:
                        existing_asset = existing_assets[0]
                        name, version = _get_existing_asset_name_and_version(existing_asset)
                        return self.get(name=name, version=version)
                    sas_info = get_storage_info_for_non_registry_asset(
                        service_client=self._service_client,
                        workspace_name=self._workspace_name,
                        name=name,
                        version=version,
                        resource_group=self._resource_group_name,
                    )
                    sas_uri = sas_info["sas_uri"]
                    blob_uri = sas_info["blob_uri"]

            code, _ = _check_and_upload_path(
                artifact=code,
                asset_operations=self,
                sas_uri=sas_uri,
                artifact_type=ErrorTarget.CODE,
                show_progress=self._show_progress,
                blob_uri=blob_uri,
            )

            # For anonymous code, if the code already exists in storage, we reuse the name,
            # version stored in the storage metadata so the same anonymous code won't be created again.
            if code._is_anonymous:
                name = code.name
                version = code.version

            code_version_resource = code._to_rest_object()

            result = (
                self._version_operation.begin_create_or_update(
                    name=name,
                    version=version,
                    registry_name=self._registry_name,
                    resource_group_name=self._operation_scope.resource_group_name,
                    body=code_version_resource,
                    **self._init_kwargs,
                ).result()
                if self._registry_name
                else self._version_operation.create_or_update(
                    name=name,
                    version=version,
                    workspace_name=self._workspace_name,
                    resource_group_name=self._operation_scope.resource_group_name,
                    body=code_version_resource,
                    **self._init_kwargs,
                )
            )

            if not result:
                return self.get(name=name, version=version)
            return Code._from_rest_object(result)
        except Exception as ex:
            if isinstance(ex, (ValidationException, SchemaValidationError)):
                log_and_raise_error(ex)
            elif isinstance(ex, HttpResponseError):
                # service side raises an exception if we attempt to update an existing asset's asset path
                if str(ex) == ASSET_PATH_ERROR:
                    raise AssetPathException(
                        message=CHANGED_ASSET_PATH_MSG,
                        target=ErrorTarget.CODE,
                        no_personal_data_message=CHANGED_ASSET_PATH_MSG_NO_PERSONAL_DATA,
                        error_category=ErrorCategory.USER_ERROR,
                    ) from ex
            raise ex

    @monitor_with_activity(ops_logger, "Code.Get", ActivityType.PUBLICAPI)
    def get(self, name: str, version: str) -> Code:
        """Returns information about the specified code asset.

        :param name: Name of the code asset.
        :type name: str
        :param version: Version of the code asset.
        :type version: str
        :raises ~azure.ai.ml.exceptions.ValidationException: Raised if Code cannot be successfully validated.
            Details will be provided in the error message.
        :return: Code asset object.
        :rtype: ~azure.ai.ml.entities.Code

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START code_operations_get]
                :end-before: [END code_operations_get]
                :language: python
                :dedent: 8
                :caption: Get code asset example.
        """
        return self._get(name=name, version=version)

    # this is a public API but CodeOperations is hidden, so it may only monitor internal calls
    @monitor_with_activity(ops_logger, "Code.Download", ActivityType.PUBLICAPI)
    def download(self, name: str, version: str, download_path: Union[PathLike, str]) -> None:
        """Download content of a code.

        :param str name: Name of the code.
        :param str version: Version of the code.
        :param Union[PathLike, str] download_path: Local path as download destination,
            defaults to current working directory of the current user. Contents will be overwritten.
        :raise: ResourceNotFoundError if can't find a code matching provided name.
        """
        output_dir = Path(download_path)
        if output_dir.is_dir():
            # an OSError will be raised if the directory is not empty
            output_dir.rmdir()
        output_dir.mkdir(parents=True)

        code = self._get(name=name, version=version)

        # TODO: how should we maintain this regex?
        m = re.match(
            r"https://(?P<account_name>.+)\.blob\.core\.windows\.net"
            r"(:[0-9]+)?/(?P<container_name>.+)/(?P<blob_name>.*)",
            str(code.path),
        )
        if not m:
            raise ValueError(f"Invalid code path: {code.path}")

        datastore_info = get_datastore_info(
            self._datastore_operation,
            # always use WORKSPACE_BLOB_STORE
            name=_get_datastore_name(),
            container_name=m.group("container_name"),
        )
        storage_client = get_storage_client(**datastore_info)
        storage_client.download(
            starts_with=m.group("blob_name"),
            destination=output_dir.as_posix(),
        )
        if not output_dir.is_dir() or not any(output_dir.iterdir()):
            raise RuntimeError(f"Failed to download code to {output_dir}")

    def _get(self, name: str, version: Optional[str] = None) -> Code:
        if not version:
            msg = "Code asset version must be specified as part of name parameter, in format 'name:version'."
            raise ValidationException(
                message=msg,
                target=ErrorTarget.CODE,
                no_personal_data_message=msg,
                error_category=ErrorCategory.USER_ERROR,
                error_type=ValidationErrorType.INVALID_VALUE,
            )
        code_version_resource = (
            self._version_operation.get(
                name=name,
                version=version,
                resource_group_name=self._operation_scope.resource_group_name,
                registry_name=self._registry_name,
                **self._init_kwargs,
            )
            if self._registry_name
            else self._version_operation.get(
                name=name,
                version=version,
                resource_group_name=self._operation_scope.resource_group_name,
                workspace_name=self._workspace_name,
                **self._init_kwargs,
            )
        )
        return Code._from_rest_object(code_version_resource)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, Optional, TYPE_CHECKING

from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from msrest import Deserializer, Serializer

from .. import models
from ._configuration import AzureMachineLearningWorkspacesConfiguration
from .operations import DataCallOperations, DataContainerOperations, DataVersionOperations, DatasetControllerV2Operations, DatasetV2Operations, DatasetsV1Operations, DeleteOperations, GetOperationStatusOperations

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential

class AzureMachineLearningWorkspaces:
    """AzureMachineLearningWorkspaces.

    :ivar data_call: DataCallOperations operations
    :vartype data_call: azure.mgmt.machinelearningservices.aio.operations.DataCallOperations
    :ivar data_container: DataContainerOperations operations
    :vartype data_container:
     azure.mgmt.machinelearningservices.aio.operations.DataContainerOperations
    :ivar delete: DeleteOperations operations
    :vartype delete: azure.mgmt.machinelearningservices.aio.operations.DeleteOperations
    :ivar datasets_v1: DatasetsV1Operations operations
    :vartype datasets_v1: azure.mgmt.machinelearningservices.aio.operations.DatasetsV1Operations
    :ivar dataset_controller_v2: DatasetControllerV2Operations operations
    :vartype dataset_controller_v2:
     azure.mgmt.machinelearningservices.aio.operations.DatasetControllerV2Operations
    :ivar dataset_v2: DatasetV2Operations operations
    :vartype dataset_v2: azure.mgmt.machinelearningservices.aio.operations.DatasetV2Operations
    :ivar data_version: DataVersionOperations operations
    :vartype data_version: azure.mgmt.machinelearningservices.aio.operations.DataVersionOperations
    :ivar get_operation_status: GetOperationStatusOperations operations
    :vartype get_operation_status:
     azure.mgmt.machinelearningservices.aio.operations.GetOperationStatusOperations
    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param base_url: Service URL. Default value is ''.
    :type base_url: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        base_url: str = "",
        **kwargs: Any
    ) -> None:
        self._config = AzureMachineLearningWorkspacesConfiguration(credential=credential, **kwargs)
        self._client = AsyncARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.data_call = DataCallOperations(self._client, self._config, self._serialize, self._deserialize)
        self.data_container = DataContainerOperations(self._client, self._config, self._serialize, self._deserialize)
        self.delete = DeleteOperations(self._client, self._config, self._serialize, self._deserialize)
        self.datasets_v1 = DatasetsV1Operations(self._client, self._config, self._serialize, self._deserialize)
        self.dataset_controller_v2 = DatasetControllerV2Operations(self._client, self._config, self._serialize, self._deserialize)
        self.dataset_v2 = DatasetV2Operations(self._client, self._config, self._serialize, self._deserialize)
        self.data_version = DataVersionOperations(self._client, self._config, self._serialize, self._deserialize)
        self.get_operation_status = GetOperationStatusOperations(self._client, self._config, self._serialize, self._deserialize)


    def _send_request(
        self,
        request: HttpRequest,
        **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/python/protocol/quickstart

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "AzureMachineLearningWorkspaces":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details) -> None:
        await self._client.__aexit__(*exc_details)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
# pylint: disable=protected-access

import re
from typing import Optional, cast

from azure.ai.ml._restclient.v2022_10_01_preview.models import ScriptReference as RestScriptReference
from azure.ai.ml._restclient.v2022_10_01_preview.models import ScriptsToExecute as RestScriptsToExecute
from azure.ai.ml._restclient.v2022_10_01_preview.models import SetupScripts as RestSetupScripts
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class ScriptReference(RestTranslatableMixin):
    """Script reference.

    :keyword path: The location of scripts in workspace storage.
    :paramtype path: Optional[str]
    :keyword command: Command line arguments passed to the script to run.
    :paramtype command: Optional[str]
    :keyword timeout_minutes: Timeout, in minutes, for the script to run.
    :paramtype timeout_minutes: Optional[int]
    """

    def __init__(
        self, *, path: Optional[str] = None, command: Optional[str] = None, timeout_minutes: Optional[int] = None
    ) -> None:
        self.path = path
        self.command = command
        self.timeout_minutes = timeout_minutes

    def _to_rest_object(self) -> RestScriptReference:
        return RestScriptReference(
            script_source="workspaceStorage",
            script_data=self.path,
            script_arguments=self.command,
            timeout=f"{self.timeout_minutes}m",
        )

    @classmethod
    def _from_rest_object(cls, obj: RestScriptReference) -> Optional["ScriptReference"]:
        if obj is None:
            return obj
        timeout_match = re.match(r"(\d+)m", obj.timeout) if obj.timeout else None
        timeout_minutes = timeout_match.group(1) if timeout_match else None
        script_reference = ScriptReference(
            path=obj.script_data if obj.script_data else None,
            command=obj.script_arguments if obj.script_arguments else None,
            timeout_minutes=cast(Optional[int], timeout_minutes),
        )
        return script_reference


class SetupScripts(RestTranslatableMixin):
    """Customized setup scripts.

    :keyword startup_script: The script to be run every time the compute is started.
    :paramtype startup_script: Optional[~azure.ai.ml.entities.ScriptReference]
    :keyword creation_script: The script to be run only when the compute is created.
    :paramtype creation_script: Optional[~azure.ai.ml.entities.ScriptReference]
    """

    def __init__(
        self, *, startup_script: Optional[ScriptReference] = None, creation_script: Optional[ScriptReference] = None
    ) -> None:
        self.startup_script = startup_script
        self.creation_script = creation_script

    def _to_rest_object(self) -> RestScriptsToExecute:
        scripts_to_execute = RestScriptsToExecute(
            startup_script=self.startup_script._to_rest_object() if self.startup_script else None,
            creation_script=self.creation_script._to_rest_object() if self.creation_script else None,
        )
        return RestSetupScripts(scripts=scripts_to_execute)

    @classmethod
    def _from_rest_object(cls, obj: RestSetupScripts) -> Optional["SetupScripts"]:
        if obj is None or obj.scripts is None:
            return None
        scripts = obj.scripts
        setup_scripts = SetupScripts(
            startup_script=ScriptReference._from_rest_object(
                scripts.startup_script if scripts.startup_script else None
            ),
            creation_script=ScriptReference._from_rest_object(
                scripts.creation_script if scripts.creation_script else None
            ),
        )
        return setup_scripts



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

from pathlib import Path
from typing import Any, Dict, Optional, Union

from azure.ai.ml._azure_environments import _get_storage_endpoint_from_metadata
from azure.ai.ml._restclient.v2023_04_01_preview.models import AzureBlobDatastore as RestAzureBlobDatastore
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    AzureDataLakeGen2Datastore as RestAzureDataLakeGen2Datastore,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import AzureFileDatastore as RestAzureFileDatastore
from azure.ai.ml._restclient.v2023_04_01_preview.models import Datastore as DatastoreData
from azure.ai.ml._restclient.v2023_04_01_preview.models import DatastoreType
from azure.ai.ml._schema._datastore import AzureBlobSchema, AzureDataLakeGen2Schema, AzureFileSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, TYPE
from azure.ai.ml.entities._credentials import (
    AccountKeyConfiguration,
    CertificateConfiguration,
    SasTokenConfiguration,
    ServicePrincipalConfiguration,
)
from azure.ai.ml.entities._datastore.datastore import Datastore
from azure.ai.ml.entities._datastore.utils import from_rest_datastore_credentials
from azure.ai.ml.entities._util import load_from_dict

from ._constants import HTTPS


class AzureFileDatastore(Datastore):
    """Azure file share that is linked to an Azure ML workspace.

    :param name: Name of the datastore.
    :type name: str
    :param account_name: Name of the Azure storage account.
    :type account_name: str
    :param file_share_name: Name of the file share.
    :type file_share_name: str
    :param description: Description of the resource.
    :type description: str
    :param tags: Tag dictionary. Tags can be added, removed, and updated.
    :type tags: dict[str, str]
    :param endpoint: Endpoint to use to connect with the Azure storage account
    :type endpoint: str
    :param protocol: Protocol to use to connect with the Azure storage account
    :type protocol: str
    :param properties: The asset property dictionary.
    :type properties: dict[str, str]
    :param credentials: Credentials to use for Azure ML workspace to connect to the storage. Defaults to None.
    :type credentials: Union[~azure.ai.ml.entities.AccountKeyConfiguration,
        ~azure.ai.ml.entities.SasTokenConfiguration]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        name: str,
        account_name: str,
        file_share_name: str,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        endpoint: str = _get_storage_endpoint_from_metadata(),
        protocol: str = HTTPS,
        properties: Optional[Dict] = None,
        credentials: Optional[Union[AccountKeyConfiguration, SasTokenConfiguration]] = None,
        **kwargs: Any
    ):
        kwargs[TYPE] = DatastoreType.AZURE_FILE
        super().__init__(
            name=name, description=description, tags=tags, properties=properties, credentials=credentials, **kwargs
        )
        self.file_share_name = file_share_name
        self.account_name = account_name
        self.endpoint = endpoint
        self.protocol = protocol

    def _to_rest_object(self) -> DatastoreData:
        file_ds = RestAzureFileDatastore(
            account_name=self.account_name,
            file_share_name=self.file_share_name,
            credentials=self.credentials._to_datastore_rest_object(),
            endpoint=self.endpoint,
            protocol=self.protocol,
            description=self.description,
            tags=self.tags,
        )
        return DatastoreData(properties=file_ds)

    @classmethod
    def _load_from_dict(cls, data: Dict, context: Dict, additional_message: str, **kwargs: Any) -> "AzureFileDatastore":
        res: AzureFileDatastore = load_from_dict(AzureFileSchema, data, context, additional_message)
        return res

    @classmethod
    def _from_rest_object(cls, datastore_resource: DatastoreData) -> "AzureFileDatastore":
        properties: RestAzureFileDatastore = datastore_resource.properties
        return AzureFileDatastore(
            name=datastore_resource.name,
            id=datastore_resource.id,
            account_name=properties.account_name,
            credentials=from_rest_datastore_credentials(properties.credentials),  # type: ignore[arg-type]
            endpoint=properties.endpoint,
            protocol=properties.protocol,
            file_share_name=properties.file_share_name,
            description=properties.description,
            tags=properties.tags,
        )

    def __eq__(self, other: Any) -> bool:
        res: bool = (
            super().__eq__(other)
            and self.file_share_name == other.file_share_name
            and self.account_name == other.account_name
            and self.endpoint == other.endpoint
            and self.protocol == other.protocol
        )
        return res

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def _to_dict(self) -> Dict:
        context = {BASE_PATH_CONTEXT_KEY: Path(".").parent}
        res: dict = AzureFileSchema(context=context).dump(self)
        return res


class AzureBlobDatastore(Datastore):
    """Azure blob storage that is linked to an Azure ML workspace.

    :param name: Name of the datastore.
    :type name: str
    :param account_name: Name of the Azure storage account.
    :type account_name: str
    :param container_name: Name of the container.
    :type container_name: str
    :param description: Description of the resource.
    :type description: str
    :param tags: Tag dictionary. Tags can be added, removed, and updated.
    :type tags: dict[str, str]
    :param endpoint: Endpoint to use to connect with the Azure storage account.
    :type endpoint: str
    :param protocol: Protocol to use to connect with the Azure storage account.
    :type protocol: str
    :param properties: The asset property dictionary.
    :type properties: dict[str, str]
    :param credentials: Credentials to use for Azure ML workspace to connect to the storage.
    :type credentials: Union[~azure.ai.ml.entities.AccountKeyConfiguration,
        ~azure.ai.ml.entities.SasTokenConfiguration]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        name: str,
        account_name: str,
        container_name: str,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        endpoint: Optional[str] = None,
        protocol: str = HTTPS,
        properties: Optional[Dict] = None,
        credentials: Optional[Union[AccountKeyConfiguration, SasTokenConfiguration]] = None,
        **kwargs: Any
    ):
        kwargs[TYPE] = DatastoreType.AZURE_BLOB
        super().__init__(
            name=name, description=description, tags=tags, properties=properties, credentials=credentials, **kwargs
        )

        self.container_name = container_name
        self.account_name = account_name
        self.endpoint = endpoint if endpoint else _get_storage_endpoint_from_metadata()
        self.protocol = protocol

    def _to_rest_object(self) -> DatastoreData:
        blob_ds = RestAzureBlobDatastore(
            account_name=self.account_name,
            container_name=self.container_name,
            credentials=self.credentials._to_datastore_rest_object(),
            endpoint=self.endpoint,
            protocol=self.protocol,
            tags=self.tags,
            description=self.description,
        )
        return DatastoreData(properties=blob_ds)

    @classmethod
    def _load_from_dict(cls, data: Dict, context: Dict, additional_message: str, **kwargs: Any) -> "AzureBlobDatastore":
        res: AzureBlobDatastore = load_from_dict(AzureBlobSchema, data, context, additional_message)
        return res

    @classmethod
    def _from_rest_object(cls, datastore_resource: DatastoreData) -> "AzureBlobDatastore":
        properties: RestAzureBlobDatastore = datastore_resource.properties
        return AzureBlobDatastore(
            name=datastore_resource.name,
            id=datastore_resource.id,
            account_name=properties.account_name,
            credentials=from_rest_datastore_credentials(properties.credentials),  # type: ignore[arg-type]
            endpoint=properties.endpoint,
            protocol=properties.protocol,
            container_name=properties.container_name,
            description=properties.description,
            tags=properties.tags,
        )

    def __eq__(self, other: Any) -> bool:
        res: bool = (
            super().__eq__(other)
            and self.container_name == other.container_name
            and self.account_name == other.account_name
            and self.endpoint == other.endpoint
            and self.protocol == other.protocol
        )
        return res

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def _to_dict(self) -> Dict:
        context = {BASE_PATH_CONTEXT_KEY: Path(".").parent}
        res: dict = AzureBlobSchema(context=context).dump(self)
        return res


class AzureDataLakeGen2Datastore(Datastore):
    """Azure data lake gen 2 that is linked to an Azure ML workspace.

    :param name: Name of the datastore.
    :type name: str
    :param account_name: Name of the Azure storage account.
    :type account_name: str
    :param filesystem: The name of the Data Lake Gen2 filesystem.
    :type filesystem: str
    :param description: Description of the resource.
    :type description: str
    :param tags: Tag dictionary. Tags can be added, removed, and updated.
    :type tags: dict[str, str]
    :param endpoint: Endpoint to use to connect with the Azure storage account
    :type endpoint: str
    :param protocol: Protocol to use to connect with the Azure storage account
    :type protocol: str
    :param credentials: Credentials to use for Azure ML workspace to connect to the storage.
    :type credentials: Union[
        ~azure.ai.ml.entities.ServicePrincipalConfiguration,
        ~azure.ai.ml.entities.CertificateConfiguration

        ]
    :param properties: The asset property dictionary.
    :type properties: dict[str, str]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        name: str,
        account_name: str,
        filesystem: str,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        endpoint: str = _get_storage_endpoint_from_metadata(),
        protocol: str = HTTPS,
        properties: Optional[Dict] = None,
        credentials: Optional[Union[ServicePrincipalConfiguration, CertificateConfiguration]] = None,
        **kwargs: Any
    ):
        kwargs[TYPE] = DatastoreType.AZURE_DATA_LAKE_GEN2
        super().__init__(
            name=name, description=description, tags=tags, properties=properties, credentials=credentials, **kwargs
        )

        self.account_name = account_name
        self.filesystem = filesystem
        self.endpoint = endpoint
        self.protocol = protocol

    def _to_rest_object(self) -> DatastoreData:
        gen2_ds = RestAzureDataLakeGen2Datastore(
            account_name=self.account_name,
            filesystem=self.filesystem,
            credentials=self.credentials._to_datastore_rest_object(),
            endpoint=self.endpoint,
            protocol=self.protocol,
            description=self.description,
            tags=self.tags,
        )
        return DatastoreData(properties=gen2_ds)

    @classmethod
    def _load_from_dict(
        cls, data: Dict, context: Dict, additional_message: str, **kwargs: Any
    ) -> "AzureDataLakeGen2Datastore":
        res: AzureDataLakeGen2Datastore = load_from_dict(AzureDataLakeGen2Schema, data, context, additional_message)
        return res

    @classmethod
    def _from_rest_object(cls, datastore_resource: DatastoreData) -> "AzureDataLakeGen2Datastore":
        properties: RestAzureDataLakeGen2Datastore = datastore_resource.properties
        return AzureDataLakeGen2Datastore(
            name=datastore_resource.name,
            id=datastore_resource.id,
            account_name=properties.account_name,
            credentials=from_rest_datastore_credentials(properties.credentials),  # type: ignore[arg-type]
            endpoint=properties.endpoint,
            protocol=properties.protocol,
            filesystem=properties.filesystem,
            description=properties.description,
            tags=properties.tags,
        )

    def __eq__(self, other: Any) -> bool:
        res: bool = (
            super().__eq__(other)
            and self.filesystem == other.filesystem
            and self.account_name == other.account_name
            and self.endpoint == other.endpoint
            and self.protocol == other.protocol
        )
        return res

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def _to_dict(self) -> Dict:
        context = {BASE_PATH_CONTEXT_KEY: Path(".").parent}
        res: dict = AzureDataLakeGen2Schema(context=context).dump(self)
        return res



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import logging
import sys

from opentelemetry import trace

from azure.ai.ml._telemetry.logging_handler import AML_INTERNAL_LOGGER_NAMESPACE


def initialize_logger_info(module_logger: logging.Logger, terminator="\n") -> None:
    """Initializes the logger with INFO level and adds a StreamHandler to the logger.

    :param module_logger: The logger to be initialized.
    :type module_logger: logging.Logger
    :param terminator: The line terminator for the StreamHandler. Defaults to a newline character.
    :type terminator: str
    :return: None
    """
    module_logger.setLevel(logging.INFO)
    module_logger.propagate = False
    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(logging.INFO)
    handler.terminator = terminator
    handler.flush = sys.stderr.flush
    module_logger.addHandler(handler)


tracer = trace.get_tracer(AML_INTERNAL_LOGGER_NAMESPACE)


class OpsLogger:
    """
    A logger class for managing logging and tracing operations within a package.
    This class initializes loggers and tracers for a specified package, and provides methods to update logging filters.
    """

    def __init__(self, name: str):
        """
        This constructor sets up the package logger, module logger, and tracer for the given package name.
        :param name: The name of the package for which the logger is being initialized.
        :type name: str
        """
        self.package_logger: logging.Logger = logging.getLogger(AML_INTERNAL_LOGGER_NAMESPACE).getChild(name)
        self.package_logger.propagate = True
        self.package_tracer = tracer
        self.module_logger = logging.getLogger(name)
        self.custom_dimensions = {}

    def update_filter(self) -> None:
        """
        Update the logging filter for the package logger.
        This method attaches the filter from the parent logger to the package logger,
            as the parent's filter is not automatically propagated to the child logger.

        :return: None
        """
        # Attach filter explicitly as parent logger's filter is not propagated to child logger
        if self.package_logger.parent.filters:
            self.package_logger.addFilter(self.package_logger.parent.filters[0])



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from azure.ai.ml._exception_helper import log_and_raise_error
from azure.ai.ml._restclient.v2023_06_01_preview.models import AmlTokenComputeIdentity, MonitorServerlessSparkCompute
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationErrorType, ValidationException


class ServerlessSparkCompute:
    """Serverless Spark compute.

    :param runtime_version: The runtime version of the compute.
    :type runtime_version: str
    :param instance_type: The instance type of the compute.
    :type instance_type: str
    """

    def __init__(
        self,
        *,
        runtime_version: str,
        instance_type: str,
    ):
        self.runtime_version = runtime_version
        self.instance_type = instance_type

    def _to_rest_object(self) -> MonitorServerlessSparkCompute:
        self._validate()
        return MonitorServerlessSparkCompute(
            runtime_version=self.runtime_version,
            instance_type=self.instance_type,
            compute_identity=AmlTokenComputeIdentity(
                compute_identity_type="AmlToken",
            ),
        )

    @classmethod
    def _from_rest_object(cls, obj: MonitorServerlessSparkCompute) -> "ServerlessSparkCompute":
        return cls(
            runtime_version=obj.runtime_version,
            instance_type=obj.instance_type,
        )

    def _validate(self) -> None:
        if self.runtime_version != "3.4":
            msg = "Compute runtime version must be 3.4"
            err = ValidationException(
                message=msg,
                target=ErrorTarget.MODEL_MONITORING,
                no_personal_data_message=msg,
                error_category=ErrorCategory.USER_ERROR,
                error_type=ValidationErrorType.MISSING_FIELD,
            )
            log_and_raise_error(err)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

import time
import uuid
from typing import Dict, Iterable, Optional, cast

from marshmallow.exceptions import ValidationError as SchemaValidationError

from azure.ai.ml._exception_helper import log_and_raise_error
from azure.ai.ml._restclient.v2024_01_01_preview import AzureMachineLearningWorkspaces as ServiceClient012024Preview
from azure.ai.ml._restclient.v2024_01_01_preview.models import ComputeInstanceDataMount
from azure.ai.ml._restclient.v2024_07_01_preview import AzureMachineLearningWorkspaces as ServiceClient072024Preview
from azure.ai.ml._restclient.v2024_07_01_preview.models import Datastore as DatastoreData
from azure.ai.ml._restclient.v2024_07_01_preview.models import DatastoreSecrets, NoneDatastoreCredentials, SecretExpiry
from azure.ai.ml._scope_dependent_operations import OperationConfig, OperationScope, _ScopeDependentOperations
from azure.ai.ml._telemetry import ActivityType, monitor_with_activity
from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml._utils._logger_utils import OpsLogger
from azure.ai.ml.entities._datastore.datastore import Datastore
from azure.ai.ml.exceptions import MlException, ValidationException

ops_logger = OpsLogger(__name__)
module_logger = ops_logger.module_logger


class DatastoreOperations(_ScopeDependentOperations):
    """Represents a client for performing operations on Datastores.

    You should not instantiate this class directly. Instead, you should create MLClient and use this client via the
    property MLClient.datastores

    :param operation_scope: Scope variables for the operations classes of an MLClient object.
    :type operation_scope: ~azure.ai.ml._scope_dependent_operations.OperationScope
    :param operation_config: Common configuration for operations classes of an MLClient object.
    :type operation_config: ~azure.ai.ml._scope_dependent_operations.OperationConfig
    :param serviceclient_2024_01_01_preview: Service client to allow end users to operate on Azure Machine Learning
        Workspace resources.
    :type serviceclient_2024_01_01_preview: ~azure.ai.ml._restclient.v2023_01_01_preview.
        _azure_machine_learning_workspaces.AzureMachineLearningWorkspaces
    :param serviceclient_2024_07_01_preview: Service client to allow end users to operate on Azure Machine Learning
        Workspace resources.
    :type serviceclient_2024_07_01_preview: ~azure.ai.ml._restclient.v2024_07_01_preview.
        _azure_machine_learning_workspaces.AzureMachineLearningWorkspaces
    """

    def __init__(
        self,
        operation_scope: OperationScope,
        operation_config: OperationConfig,
        serviceclient_2024_01_01_preview: ServiceClient012024Preview,
        serviceclient_2024_07_01_preview: ServiceClient072024Preview,
        **kwargs: Dict,
    ):
        super(DatastoreOperations, self).__init__(operation_scope, operation_config)
        ops_logger.update_filter()
        self._operation = serviceclient_2024_07_01_preview.datastores
        self._compute_operation = serviceclient_2024_01_01_preview.compute
        self._credential = serviceclient_2024_07_01_preview._config.credential
        self._init_kwargs = kwargs

    @monitor_with_activity(ops_logger, "Datastore.List", ActivityType.PUBLICAPI)
    def list(self, *, include_secrets: bool = False) -> Iterable[Datastore]:
        """Lists all datastores and associated information within a workspace.

        :keyword include_secrets: Include datastore secrets in returned datastores, defaults to False
        :paramtype include_secrets: bool
        :return: An iterator like instance of Datastore objects
        :rtype: ~azure.core.paging.ItemPaged[Datastore]

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START datastore_operations_list]
                :end-before: [END datastore_operations_list]
                :language: python
                :dedent: 8
                :caption: List datastore example.
        """

        def _list_helper(datastore_resource: Datastore, include_secrets: bool) -> Datastore:
            if include_secrets:
                self._fetch_and_populate_secret(datastore_resource)
            return Datastore._from_rest_object(datastore_resource)

        return cast(
            Iterable[Datastore],
            self._operation.list(
                resource_group_name=self._operation_scope.resource_group_name,
                workspace_name=self._workspace_name,
                cls=lambda objs: [_list_helper(obj, include_secrets) for obj in objs],
                **self._init_kwargs,
            ),
        )

    @monitor_with_activity(ops_logger, "Datastore.ListSecrets", ActivityType.PUBLICAPI)
    def _list_secrets(self, name: str, expirable_secret: bool = False) -> DatastoreSecrets:
        return self._operation.list_secrets(
            name=name,
            body=SecretExpiry(expirable_secret=expirable_secret),
            resource_group_name=self._operation_scope.resource_group_name,
            workspace_name=self._workspace_name,
            **self._init_kwargs,
        )

    @monitor_with_activity(ops_logger, "Datastore.Delete", ActivityType.PUBLICAPI)
    def delete(self, name: str) -> None:
        """Deletes a datastore reference with the given name from the workspace. This method does not delete the actual
        datastore or underlying data in the datastore.

        :param name: Name of the datastore
        :type name: str

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START datastore_operations_delete]
                :end-before: [END datastore_operations_delete]
                :language: python
                :dedent: 8
                :caption: Delete datastore example.
        """

        self._operation.delete(
            name=name,
            resource_group_name=self._operation_scope.resource_group_name,
            workspace_name=self._workspace_name,
            **self._init_kwargs,
        )

    @monitor_with_activity(ops_logger, "Datastore.Get", ActivityType.PUBLICAPI)
    def get(self, name: str, *, include_secrets: bool = False) -> Datastore:  # type: ignore
        """Returns information about the datastore referenced by the given name.

        :param name: Datastore name
        :type name: str
        :keyword include_secrets: Include datastore secrets in the returned datastore, defaults to False
        :paramtype include_secrets: bool
        :return: Datastore with the specified name.
        :rtype: Datastore

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START datastore_operations_get]
                :end-before: [END datastore_operations_get]
                :language: python
                :dedent: 8
                :caption: Get datastore example.
        """
        try:
            datastore_resource = self._operation.get(
                name=name,
                resource_group_name=self._operation_scope.resource_group_name,
                workspace_name=self._workspace_name,
                **self._init_kwargs,
            )
            if include_secrets:
                self._fetch_and_populate_secret(datastore_resource)
            return Datastore._from_rest_object(datastore_resource)
        except (ValidationException, SchemaValidationError) as ex:
            log_and_raise_error(ex)

    def _fetch_and_populate_secret(self, datastore_resource: DatastoreData) -> None:
        if datastore_resource.name and not isinstance(
            datastore_resource.properties.credentials, NoneDatastoreCredentials
        ):
            secrets = self._list_secrets(name=datastore_resource.name, expirable_secret=True)
            datastore_resource.properties.credentials.secrets = secrets

    @monitor_with_activity(ops_logger, "Datastore.GetDefault", ActivityType.PUBLICAPI)
    def get_default(self, *, include_secrets: bool = False) -> Datastore:  # type: ignore
        """Returns the workspace's default datastore.

        :keyword include_secrets: Include datastore secrets in the returned datastore, defaults to False
        :paramtype include_secrets: bool
        :return: The default datastore.
        :rtype: Datastore

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START datastore_operations_get_default]
                :end-before: [END datastore_operations_get_default]
                :language: python
                :dedent: 8
                :caption: Get default datastore example.
        """
        try:
            datastore_resource = self._operation.list(
                resource_group_name=self._operation_scope.resource_group_name,
                workspace_name=self._workspace_name,
                is_default=True,
                **self._init_kwargs,
            ).next()
            if include_secrets:
                self._fetch_and_populate_secret(datastore_resource)
            return Datastore._from_rest_object(datastore_resource)
        except (ValidationException, SchemaValidationError) as ex:
            log_and_raise_error(ex)

    @monitor_with_activity(ops_logger, "Datastore.CreateOrUpdate", ActivityType.PUBLICAPI)
    def create_or_update(self, datastore: Datastore) -> Datastore:  # type: ignore
        """Attaches the passed in datastore to the workspace or updates the datastore if it already exists.

        :param datastore: The configuration of the datastore to attach.
        :type datastore: Datastore
        :return: The attached datastore.
        :rtype: Datastore

        .. admonition:: Example:

            .. literalinclude:: ../samples/ml_samples_misc.py
                :start-after: [START datastore_operations_create_or_update]
                :end-before: [END datastore_operations_create_or_update]
                :language: python
                :dedent: 8
                :caption: Create datastore example.
        """
        try:
            ds_request = datastore._to_rest_object()
            datastore_resource = self._operation.create_or_update(
                name=datastore.name,
                resource_group_name=self._operation_scope.resource_group_name,
                workspace_name=self._workspace_name,
                body=ds_request,
                skip_validation=True,
            )
            return Datastore._from_rest_object(datastore_resource)
        except Exception as ex:  # pylint: disable=W0718
            if isinstance(ex, (ValidationException, SchemaValidationError)):
                log_and_raise_error(ex)
            else:
                raise ex

    @monitor_with_activity(ops_logger, "Datastore.Mount", ActivityType.PUBLICAPI)
    @experimental
    def mount(
        self,
        path: str,
        *,
        mount_point: Optional[str] = None,
        mode: str = "ro_mount",
        debug: bool = False,
        persistent: bool = False,
        **kwargs,
    ) -> None:
        """Mount a datastore to a local path, so that you can access data inside it
        under a local path with any tools of your choice.

        :param path: The data store path to mount, in the form of `<name>` or `azureml://datastores/<name>`.
        :type path: str
        :keyword mount_point: A local path used as mount point.
        :type mount_point: str
        :keyword mode: Mount mode, either `ro_mount` (read-only) or `rw_mount` (read-write).
        :type mode: str
        :keyword debug: Whether to enable verbose logging.
        :type debug: bool
        :keyword persistent: Whether to persist the mount after reboot. Applies only when running on Compute Instance,
                where the 'CI_NAME' environment variable is set."
        :type persistent: bool
        :return: None
        """

        assert mode in ["ro_mount", "rw_mount"], "mode should be either `ro_mount` or `rw_mount`"
        read_only = mode == "ro_mount"

        import os

        ci_name = os.environ.get("CI_NAME")
        assert not persistent or (
            persistent and ci_name is not None
        ), "persistent mount is only supported on Compute Instance, where the 'CI_NAME' environment variable is set."

        try:
            from azureml.dataprep import rslex_fuse_subprocess_wrapper
        except ImportError as exc:
            msg = "Mount operations requires package azureml-dataprep-rslex installed. \
                You can install it with Azure ML SDK with `pip install azure-ai-ml[mount]`."
            raise MlException(message=msg, no_personal_data_message=msg) from exc

        uri = rslex_fuse_subprocess_wrapper.build_datastore_uri(
            self._operation_scope._subscription_id, self._resource_group_name, self._workspace_name, path
        )
        if persistent and ci_name is not None:
            mount_name = f"unified_mount_{str(uuid.uuid4()).replace('-', '')}"
            self._compute_operation.update_data_mounts(
                self._resource_group_name,
                self._workspace_name,
                ci_name,
                [
                    ComputeInstanceDataMount(
                        source=uri,
                        source_type="URI",
                        mount_name=mount_name,
                        mount_action="Mount",
                        mount_path=mount_point or "",
                    )
                ],
                api_version="2021-01-01",
                **kwargs,
            )
            print(f"Mount requested [name: {mount_name}]. Waiting for completion ...")
            while True:
                compute = self._compute_operation.get(self._resource_group_name, self._workspace_name, ci_name)
                mounts = compute.properties.properties.data_mounts
                try:
                    mount = [mount for mount in mounts if mount.mount_name == mount_name][0]
                    if mount.mount_state == "Mounted":
                        print(f"Mounted [name: {mount_name}].")
                        break
                    if mount.mount_state == "MountRequested":
                        pass
                    elif mount.mount_state == "MountFailed":
                        msg = f"Mount failed [name: {mount_name}]: {mount.error}"
                        raise MlException(message=msg, no_personal_data_message=msg)
                    else:
                        msg = f"Got unexpected mount state [name: {mount_name}]: {mount.mount_state}"
                        raise MlException(message=msg, no_personal_data_message=msg)
                except IndexError:
                    pass
                time.sleep(5)
        else:
            rslex_fuse_subprocess_wrapper.start_fuse_mount_subprocess(
                uri, mount_point, read_only, debug, credential=self._operation._config.credential
            )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


import logging
from concurrent.futures import Future
from time import time
from typing import Optional, Union

from azure.ai.ml.constants._common import LROConfigurations
from azure.core.polling import LROPoller
from azure.mgmt.core.polling.arm_polling import ARMPolling

module_logger = logging.getLogger(__name__)


class AzureMLPolling(ARMPolling):
    """A polling class for azure machine learning."""

    def update_status(self):
        """Update the current status of the LRO."""
        super().update_status()
        print(".", end="", flush=True)


def polling_wait(
    poller: Union[LROPoller, Future],
    message: Optional[str] = None,
    start_time: Optional[float] = None,
) -> None:
    """Print out status while polling and time of operation once completed.

    :param poller: An poller which will return status update via function done().
    :type poller: Union[LROPoller, concurrent.futures.Future]
    :param message: Message to print out before starting operation write-out.
    :type message: Optional[str]
    :param start_time: Start time of operation.
    :type start_time: Optional[float]
    """
    module_logger.info("%s", message)
    poller.result(timeout=LROConfigurations.POLLING_TIMEOUT)
    module_logger.info("Done.")

    if start_time:
        end_time = time()
        duration = divmod(int(round(end_time - start_time)), 60)
        module_logger.info("(%sm %ss)\n", duration[0], duration[1])



from pathlib import Path

import pytest
import yaml
from marshmallow.exceptions import ValidationError
from test_utilities.utils import verify_entity_load_and_dump

from azure.ai.ml import load_model
from azure.ai.ml._restclient.v2021_10_01_dataplanepreview.models import ModelVersionData, EnvironmentVersionData
from azure.ai.ml._schema import ModelSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, AssetTypes
from azure.ai.ml.entities._assets import Model, Environment
from azure.ai.ml.entities._util import load_from_dict
from azure.ai.ml.entities._load_functions import load_model, load_model_package, load_online_deployment


@pytest.mark.unittest
@pytest.mark.production_experiences_test
class TestModelPackageSchema:
    def test_model_package(self) -> None:
        file = Path("tests/test_configs/model_package/model_package_simple.yml")
        model_pack = load_model_package(source=file)
        assert model_pack.name == "diabetes-online-mlflow"
        assert model_pack.inferencing_server.type == "azureml_online"
        assert model_pack.model_configuration.mode == "download"

    def test_model_package_with_online_deployment(self) -> None:
        file = Path("tests/test_configs/model_package/online_deployment_registry_package.yml")
        deployment = load_online_deployment(source=file)
        assert deployment.environment == "azureml://registries/bani-euap/environments/testv1/versions/20"
        assert deployment.model == "azureml://registries/bani-euap/models/demo_ml/versions/1"

    def test_model_package_schema(self) -> None:
        file = Path("tests/test_configs/model_package/model_package_config_copy.yml")
        deployment = load_model_package(source=file)
        assert deployment.model_configuration.mode == "copy"



from pathlib import Path

import pytest

from azure.ai.ml import Input, dsl, load_component

tests_root_dir = Path(__file__).parent.parent.parent
components_dir = tests_root_dir / "test_configs/components/"


@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestPipelineBuilder:
    def test_simple(self):
        component_yaml = components_dir / "helloworld_component_optional_input.yml"
        component_func = load_component(component_yaml)

        @dsl.pipeline
        def pipeline_func(required_input: int, optional_input: int = 2):
            named_step = component_func(required_input=required_input, optional_input=optional_input)

        pipeline_job = pipeline_func(1, 2)
        assert "named_step" in pipeline_job.jobs

    def test_raise_exception(self):
        @dsl.pipeline
        def mock_error_exception():
            mock_local_variable = 1
            return mock_local_variable / 0

        with pytest.raises(ZeroDivisionError):
            mock_error_exception()

    def test_instance_func(self):
        component_yaml = components_dir / "helloworld_component_optional_input.yml"
        component_func = load_component(component_yaml)

        class MockClass(Input):
            def __init__(self, mock_path):
                super(MockClass, self).__init__(path=mock_path)

            @dsl.pipeline
            def pipeline_func(self, required_input: int, optional_input: int = 2):
                named_step = component_func(required_input=required_input, optional_input=optional_input)

        mock_obj = MockClass("./some/path")
        pipeline_job = mock_obj.pipeline_func(1, 2)
        assert "named_step" in pipeline_job.jobs
        assert "self" in pipeline_job.inputs
        assert pipeline_job.inputs["self"].path == "./some/path"

    def test_node_as_input_output(self):
        component_yaml = components_dir / "helloworld_component.yml"
        component_func = load_component(component_yaml)

        @dsl.pipeline
        def base_pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = component_func(component_in_path=input_path, component_in_number=input_number)
            node2 = component_func(
                component_in_path=node1.outputs["component_out_path"],
                component_in_number=input_number,
            )
            # return {
            #     'component_out_path': node2.outputs['component_out_path'],
            # }

        @dsl.pipeline
        def pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = component_func(component_in_path=input_path, component_in_number=input_number)
            # single output node as node input
            node2 = component_func(
                component_in_path=node1,
                component_in_number=input_number,
            )
            # single output node as pipeline output
            # return node2

        base_pipeline_job = base_pipeline_func(Input(path="./tests/test_configs/data"), 0.5)
        pipeline_job = pipeline_func(Input(path="./tests/test_configs/data"), 0.5)
        pipeline_job.display_name = base_pipeline_job.display_name
        assert (
            pipeline_job._to_rest_object().properties.as_dict()
            == base_pipeline_job._to_rest_object().properties.as_dict()
        )

    def test_node_as_sub_pipeline_input(self):
        component_yaml = components_dir / "helloworld_component.yml"
        component_func = load_component(component_yaml)

        @dsl.pipeline
        def sub_pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = component_func(component_in_path=input_path, component_in_number=input_number)
            node2 = component_func(
                component_in_path=node1.outputs["component_out_path"],
                component_in_number=input_number,
            )
            return {
                "component_out_path": node2.outputs["component_out_path"],
            }

        @dsl.pipeline
        def base_pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = component_func(component_in_path=input_path, component_in_number=input_number)
            node2 = sub_pipeline_func(
                input_path=node1.outputs["component_out_path"],
                input_number=input_number,
            )
            # return {
            #     'component_out_path': node2.outputs['component_out_path'],
            # }

        @dsl.pipeline
        def pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = component_func(component_in_path=input_path, component_in_number=input_number)
            # single output node as node input
            node2 = sub_pipeline_func(
                input_path=node1,
                input_number=input_number,
            )
            # single output node as pipeline output
            # return node2

        base_pipeline_job = base_pipeline_func(Input(path="./tests/test_configs/data"), 0.5)
        pipeline_job = pipeline_func(Input(path="./tests/test_configs/data"), 0.5)
        pipeline_job.display_name = base_pipeline_job.display_name
        assert (
            pipeline_job._to_rest_object().properties.as_dict()
            == base_pipeline_job._to_rest_object().properties.as_dict()
        )

    def test_node_as_sub_pipeline_input_error(self):
        single_output_component_func = load_component(components_dir / "helloworld_component.yml")
        multi_output_component_func = load_component(components_dir / "helloworld_component_multi_outputs.yml")

        @dsl.pipeline
        def sub_pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = single_output_component_func(component_in_path=input_path, component_in_number=input_number)
            node2 = single_output_component_func(
                component_in_path=node1.outputs["component_out_path"],
                component_in_number=input_number,
            )
            return {
                "component_out_path": node2.outputs["component_out_path"],
            }

        @dsl.pipeline
        def pipeline_func(input_path: Input, input_number: float = 0.5):
            node1 = multi_output_component_func(component_in_path=input_path, component_in_number=input_number)
            # single output node as node input
            node2 = sub_pipeline_func(
                input_path=node1,
                input_number=input_number,
            )
            # single output node as pipeline output
            # return node2

        with pytest.raises(
            ValueError, match="Provided input input_path is not a single output node, cannot be used as a node input."
        ):
            pipeline_func(Input(path="./tests/test_configs/data"), 0.5)

    def test_with_global_condition(self):
        single_output_component_func = load_component(components_dir / "helloworld_component.yml")

        is_true = True

        @dsl.pipeline
        def sub_components(input_path: Input):
            if is_true:
                node = single_output_component_func(component_in_path=input_path, component_in_number=0.2)
            else:
                node = single_output_component_func(component_in_path=input_path, component_in_number=0.7)
            return {"component_out_path": node.outputs["component_out_path"]}

        @dsl.pipeline
        def pipeline_func(input_path: Input):
            sub_components(input_path=input_path)

        pipeline_job = pipeline_func(Input(path="./tests/test_configs/data"))
        assert len(pipeline_job.jobs) == 1
        assert pipeline_job.jobs["sub_components"].component.jobs["node"].inputs["component_in_number"]._data == 0.2

        is_true = False
        pipeline_job = pipeline_func(Input(path="./tests/test_configs/data"))
        assert len(pipeline_job.jobs) == 1
        assert pipeline_job.jobs["sub_components"].component.jobs["node"].inputs["component_in_number"]._data == 0.7



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import os
from typing import Any, Dict, List, Optional, Union, cast

from marshmallow import Schema

from azure.ai.ml._schema.component.command_component import CommandComponentSchema
from azure.ai.ml.constants._common import COMPONENT_TYPE
from azure.ai.ml.constants._component import NodeType
from azure.ai.ml.entities._assets import Environment
from azure.ai.ml.entities._job.distribution import (
    DistributionConfiguration,
    MpiDistribution,
    PyTorchDistribution,
    RayDistribution,
    TensorFlowDistribution,
)
from azure.ai.ml.entities._job.job_resource_configuration import JobResourceConfiguration
from azure.ai.ml.entities._job.parameterized_command import ParameterizedCommand
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException

from ..._restclient.v2022_10_01.models import ComponentVersion
from ..._schema import PathAwareSchema
from ..._utils.utils import get_all_data_binding_expressions, parse_args_description_from_docstring
from .._util import convert_ordered_dict_to_dict, validate_attribute_type
from .._validation import MutableValidationResult
from ._additional_includes import AdditionalIncludesMixin
from .component import Component

# pylint: disable=protected-access


class CommandComponent(Component, ParameterizedCommand, AdditionalIncludesMixin):
    """Command component version, used to define a Command Component or Job.

    :keyword name: The name of the Command job or component.
    :paramtype name: Optional[str]
    :keyword version: The version of the Command job or component.
    :paramtype version: Optional[str]
    :keyword description: The description of the component. Defaults to None.
    :paramtype description: Optional[str]
    :keyword tags: Tag dictionary. Tags can be added, removed, and updated. Defaults to None.
    :paramtype tags: Optional[dict]
    :keyword display_name: The display name of the component.
    :paramtype display_name: Optional[str]
    :keyword command: The command to be executed.
    :paramtype command: Optional[str]
    :keyword code: The source code to run the job. Can be a local path or "http:", "https:", or "azureml:" url pointing
        to a remote location.
    :type code: Optional[str]
    :keyword environment: The environment that the job will run in.
    :paramtype environment: Optional[Union[str, ~azure.ai.ml.entities.Environment]]
    :keyword distribution: The configuration for distributed jobs. Defaults to None.
    :paramtype distribution: Optional[Union[~azure.ai.ml.PyTorchDistribution, ~azure.ai.ml.MpiDistribution,
        ~azure.ai.ml.TensorFlowDistribution, ~azure.ai.ml.RayDistribution]]
    :keyword resources: The compute resource configuration for the command.
    :paramtype resources: Optional[~azure.ai.ml.entities.JobResourceConfiguration]
    :keyword inputs: A mapping of input names to input data sources used in the job. Defaults to None.
    :paramtype inputs: Optional[dict[str, Union[
        ~azure.ai.ml.Input,
        str,
        bool,
        int,
        float,
        Enum,
        ]]]
    :keyword outputs: A mapping of output names to output data sources used in the job. Defaults to None.
    :paramtype outputs: Optional[dict[str, Union[str, ~azure.ai.ml.Output]]]
    :keyword instance_count: The number of instances or nodes to be used by the compute target. Defaults to 1.
    :paramtype instance_count: Optional[int]
    :keyword is_deterministic: Specifies whether the Command will return the same output given the same input.
        Defaults to True. When True, if a Command (component) is deterministic and has been run before in the
        current workspace with the same input and settings, it will reuse results from a previous submitted job
        when used as a node or step in a pipeline. In that scenario, no compute resources will be used.
    :paramtype is_deterministic: Optional[bool]
    :keyword additional_includes: A list of shared additional files to be included in the component. Defaults to None.
    :paramtype additional_includes: Optional[List[str]]
    :keyword properties: The job property dictionary. Defaults to None.
    :paramtype properties: Optional[dict[str, str]]
    :raises ~azure.ai.ml.exceptions.ValidationException: Raised if CommandComponent cannot be successfully validated.
        Details will be provided in the error message.

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_command_configurations.py
            :start-after: [START command_component_definition]
            :end-before: [END command_component_definition]
            :language: python
            :dedent: 8
            :caption: Creating a CommandComponent.
    """

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        version: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        display_name: Optional[str] = None,
        command: Optional[str] = None,
        code: Optional[Union[str, os.PathLike]] = None,
        environment: Optional[Union[str, Environment]] = None,
        distribution: Optional[
            Union[
                Dict,
                MpiDistribution,
                TensorFlowDistribution,
                PyTorchDistribution,
                RayDistribution,
                DistributionConfiguration,
            ]
        ] = None,
        resources: Optional[JobResourceConfiguration] = None,
        inputs: Optional[Dict] = None,
        outputs: Optional[Dict] = None,
        instance_count: Optional[int] = None,  # promoted property from resources.instance_count
        is_deterministic: bool = True,
        additional_includes: Optional[List] = None,
        properties: Optional[Dict] = None,
        **kwargs: Any,
    ) -> None:
        # validate init params are valid type
        validate_attribute_type(attrs_to_check=locals(), attr_type_map=self._attr_type_map())

        kwargs[COMPONENT_TYPE] = NodeType.COMMAND

        # Component backend doesn't support environment_variables yet,
        # this is to support the case of CommandComponent being the trial of
        # a SweepJob, where environment_variables is stored as part of trial
        environment_variables = kwargs.pop("environment_variables", None)
        super().__init__(
            name=name,
            version=version,
            description=description,
            tags=tags,
            display_name=display_name,
            inputs=inputs,
            outputs=outputs,
            is_deterministic=is_deterministic,
            properties=properties,
            **kwargs,
        )

        # No validation on value passed here because in pipeline job, required code&environment maybe absent
        # and fill in later with job defaults.
        self.command = command
        self.code = code
        self.environment_variables = environment_variables
        self.environment = environment
        self.resources = resources  # type: ignore[assignment]
        self.distribution = distribution

        # check mutual exclusivity of promoted properties
        if self.resources is not None and instance_count is not None:
            msg = "instance_count and resources are mutually exclusive"
            raise ValidationException(
                message=msg,
                target=ErrorTarget.COMPONENT,
                no_personal_data_message=msg,
                error_category=ErrorCategory.USER_ERROR,
            )
        self.instance_count = instance_count
        self.additional_includes = additional_includes or []

    def _to_ordered_dict_for_yaml_dump(self) -> Dict:
        """Dump the component content into a sorted yaml string.

        :return: The ordered dict
        :rtype: Dict
        """

        obj: dict = super()._to_ordered_dict_for_yaml_dump()
        # dict dumped base on schema will transfer code to an absolute path, while we want to keep its original value
        if self.code and isinstance(self.code, str):
            obj["code"] = self.code
        return obj

    @property
    def instance_count(self) -> Optional[int]:
        """The number of instances or nodes to be used by the compute target.

        :return: The number of instances or nodes.
        :rtype: int
        """
        return self.resources.instance_count if self.resources and not isinstance(self.resources, dict) else None

    @instance_count.setter
    def instance_count(self, value: int) -> None:
        """Sets the number of instances or nodes to be used by the compute target.

        :param value: The number of instances of nodes to be used by the compute target. Defaults to 1.
        :type value: int
        """
        if not value:
            return
        if not self.resources:
            self.resources = JobResourceConfiguration(instance_count=value)
        else:
            if not isinstance(self.resources, dict):
                self.resources.instance_count = value

    @classmethod
    def _attr_type_map(cls) -> dict:
        return {
            "environment": (str, Environment),
            "environment_variables": dict,
            "resources": (dict, JobResourceConfiguration),
            "code": (str, os.PathLike),
        }

    def _to_dict(self) -> Dict:
        return cast(
            dict, convert_ordered_dict_to_dict({**self._other_parameter, **super(CommandComponent, self)._to_dict()})
        )

    @classmethod
    def _from_rest_object_to_init_params(cls, obj: ComponentVersion) -> Dict:
        # put it here as distribution is shared by some components, e.g. command
        distribution = obj.properties.component_spec.pop("distribution", None)
        init_kwargs: dict = super()._from_rest_object_to_init_params(obj)
        if distribution:
            init_kwargs["distribution"] = DistributionConfiguration._from_rest_object(distribution)
        return init_kwargs

    def _get_environment_id(self) -> Union[str, None]:
        # Return environment id of environment
        # handle case when environment is defined inline
        if isinstance(self.environment, Environment):
            _id: Optional[str] = self.environment.id
            return _id
        return self.environment

    # region SchemaValidatableMixin
    @classmethod
    def _create_schema_for_validation(cls, context: Any) -> Union[PathAwareSchema, Schema]:
        return CommandComponentSchema(context=context)

    def _customized_validate(self) -> MutableValidationResult:
        validation_result = super(CommandComponent, self)._customized_validate()
        self._append_diagnostics_and_check_if_origin_code_reliable_for_local_path_validation(validation_result)
        validation_result.merge_with(self._validate_command())
        validation_result.merge_with(self._validate_early_available_output())
        return validation_result

    def _validate_command(self) -> MutableValidationResult:
        validation_result = self._create_empty_validation_result()
        # command
        if self.command:
            invalid_expressions = []
            for data_binding_expression in get_all_data_binding_expressions(self.command, is_singular=False):
                if not self._is_valid_data_binding_expression(data_binding_expression):
                    invalid_expressions.append(data_binding_expression)

            if invalid_expressions:
                validation_result.append_error(
                    yaml_path="command",
                    message="Invalid data binding expression: {}".format(", ".join(invalid_expressions)),
                )
        return validation_result

    def _validate_early_available_output(self) -> MutableValidationResult:
        validation_result = self._create_empty_validation_result()
        for name, output in self.outputs.items():
            if output.early_available is True and output._is_primitive_type is not True:
                msg = (
                    f"Early available output {name!r} requires output is primitive type, "
                    f"got {output._is_primitive_type!r}."
                )
                validation_result.append_error(message=msg, yaml_path=f"outputs.{name}")
        return validation_result

    def _is_valid_data_binding_expression(self, data_binding_expression: str) -> bool:
        current_obj: Any = self
        for item in data_binding_expression.split("."):
            if hasattr(current_obj, item):
                current_obj = getattr(current_obj, item)
            else:
                try:
                    current_obj = current_obj[item]
                except Exception:  # pylint: disable=W0718
                    return False
        return True

    # endregion

    @classmethod
    def _parse_args_description_from_docstring(cls, docstring: str) -> Dict:
        res: dict = parse_args_description_from_docstring(docstring)
        return res

    def __str__(self) -> str:
        try:
            toYaml: str = self._to_yaml()
            return toYaml
        except BaseException:  # pylint: disable=W0718
            toStr: str = super(CommandComponent, self).__str__()
            return toStr



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__("pkgutil").extend_path(__path__, __name__)


from .batch.batch_endpoint import BatchEndpointSchema
from .online.online_endpoint import KubernetesOnlineEndpointSchema, ManagedOnlineEndpointSchema

__all__ = [
    "BatchEndpointSchema",
    "KubernetesOnlineEndpointSchema",
    "ManagedOnlineEndpointSchema",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from pathlib import Path
from typing import Callable

import pytest
from devtools_testutils import AzureRecordedTestCase
from test_utilities.utils import verify_entity_load_and_dump

from azure.ai.ml import MLClient
from azure.ai.ml.entities._load_functions import load_feature_store_entity

from azure.ai.ml.entities._feature_store_entity.feature_store_entity import FeatureStoreEntity
from azure.core.paging import ItemPaged
from azure.core.polling import LROPoller


@pytest.mark.e2etest
@pytest.mark.data_experiences_test
@pytest.mark.usefixtures("recorded_test", "mock_code_hash")
class TestFeatureStoreEntity(AzureRecordedTestCase):
    def test_create_and_get(self, feature_store_client: MLClient, tmp_path: Path, randstr: Callable[[], str]) -> None:
        fs_entity_name = f"e2etest_{randstr('fs_entity_name')}"
        fs_entity_description = "Feature store entity description"
        version = "1"

        params_override = [
            {"name": fs_entity_name},
            {"version": version},
            {"description": fs_entity_description},
        ]

        def feature_store_entitiy_validation(fs_entity):
            fs_entity_poller = feature_store_client.feature_store_entities.begin_create_or_update(
                feature_store_entity=fs_entity
            )
            assert isinstance(fs_entity_poller, LROPoller)
            fs_entity = fs_entity_poller.result()
            assert isinstance(fs_entity, FeatureStoreEntity)
            assert fs_entity.name == fs_entity_name
            assert fs_entity.description == fs_entity_description

        featureStoreEntity = verify_entity_load_and_dump(
            load_feature_store_entity,
            feature_store_entitiy_validation,
            "./tests/test_configs/feature_store_entity/feature_store_entity_full.yaml",
            params_override=params_override,
        )[0]

        entity_list = feature_store_client.feature_store_entities.list(name=fs_entity_name)
        assert isinstance(entity_list, ItemPaged)

        fs_entity = feature_store_client.feature_store_entities.get(name=fs_entity_name, version=version)
        assert isinstance(fs_entity, FeatureStoreEntity)
        assert fs_entity.name == fs_entity_name



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import yaml
import pytest

from azure.ai.ml.entities._feature_store_entity.feature_store_entity import FeatureStoreEntity

from azure.ai.ml.entities._load_functions import load_feature_store_entity


@pytest.mark.unittest
@pytest.mark.data_experiences_test
class TestFeatureStoreEntitySchema:
    def test_feature_store_entity_load(self) -> None:
        test_path = "./tests/test_configs/feature_store_entity/feature_store_entity_full.yaml"
        with open(test_path, "r") as f:
            target = yaml.safe_load(f)
        with open(test_path, "r") as f:
            feature_store_entity: FeatureStoreEntity = load_feature_store_entity(source=test_path)
        assert feature_store_entity.name == target["name"]
        assert feature_store_entity.version == target["version"]
        assert feature_store_entity.description == target["description"]
        assert feature_store_entity.index_columns is not None
        assert feature_store_entity.tags is not None
        assert feature_store_entity.properties is not None
